# pylint: disable=no-member,too-many-public-methods
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Configuration handling helpers.
"""


import os
import json
from six import string_types
from pydash import get
from jsonmerge import merge

import icetea_lib.LogManager as LogManager
from icetea_lib.TestStepError import TestStepError
from icetea_lib.TestStepError import InconclusiveError
from icetea_lib.tools.tools import find_duplicate_keys


class Configurations(object):
    """
    Configurations manage test and environment configurations.
    Provide public API's to read configuration values.
    """
    def __init__(self, args=None, logger=None, **kwargs):
        super(Configurations, self).__init__()
        self._config, self._integer_keys_found = self._parse_config(**kwargs)
        self._env_cfg = None
        self._logger = logger if logger else LogManager.get_dummy_logger()
        self._args = args

    def init(self, logger):
        """
        When test starts Bench will call _init() -function to initialize
        configurations and read execution configuration file if exists.
        :param args: arguments
        """
        self._logger = logger
        if self._integer_keys_found:
            self._logger.warning("Integer keys found in configuration for DUT requirements. "
                                 "Keys forced to strings for this run. "
                                 "Please update your DUT requirements keys to strings.")
        # Read cli given environment configuration file
        self._env_cfg = self._read_env_configs(self._args.env_cfg, self._args.iface)
        # Read cli given TC configuration file and merge it
        self._read_exec_configs(self._args)

    @property
    def test_name(self):
        """
        Get test bench name
        :return: string
        """
        # Return unknown also when name is set as None
        return "unknown" if self._config.get("name",
                                             "unknown") is None else self._config.get("name",
                                                                                      "unknown")

    @property
    def name(self):
        """
        Get test bench name
        :return: string
        """
        return self.test_name

    @property
    def config(self):
        """
        Getter for the internal config dict.

        :return: dict
        """
        return self._config

    @config.setter
    def config(self, value):
        self.set_config(value)

    @property
    def env(self):
        """
        Getter for env configuration

        :return: dict
        """
        return self._env_cfg

    def is_hardware_in_use(self):
        """
        :return: True if type is hardware
        """
        return get(self.config, "requirements.duts.*.type") == "hardware"

    def get_test_component(self):
        """
        Get test component.

        :return: string
        """
        return self.config.get("component", [''])

    def get_features_under_test(self):
        """
        Get features tested by this test case.

        :return: list
        """
        fea = self.config.get("feature", [])
        if isinstance(fea, str):
            return [fea]
        return fea

    def get_allowed_platforms(self):
        """
        Return list of allowed platfroms from requirements.

        :return: list
        """
        return get(self.config, "requirements.duts.*.allowed_platforms", list())

    # @todo find better place for these
    def status(self):
        """
        Get TC implementation status.

        :return: string or None
        """
        return self.config.get('status')

    def type(self):
        """
        Get test case type.

        :return: string or None
        """
        return self.config.get('type')

    def subtype(self):
        """
        Get test case subtype.

        :return: string or None
        """
        return self.config.get('subtype')

    def get_config(self):
        """
        Get test case configuration.

        :return: dict
        """
        return self.config

    def skip(self):
        """
        Get skip value.

        :return: Boolean or None
        """
        return get(self.config, "execution.skip.value")

    def skip_info(self):
        """
        Get the entire skip dictionary.

        :return: dictionary or None
        """
        return get(self.config, "execution.skip")

    # Get Skip Reason
    def skip_reason(self):
        """
        Get skip reason.

        :return: string
        """
        return get(self.config, "execution.skip.reason", "")

    def check_skip(self):
        """
        Check if tc should be skipped

        :return: Boolean
        """
        if not self.skip():
            return False

        info = self.skip_info()
        only_type = info.get('only_type')
        platforms = info.get('platforms', [])

        allowed = get(self.config, "requirements.duts.*.allowed_platforms", list())
        # validate platforms in allowed_platforms, otherwise no skip
        if allowed and platforms and not set(platforms).issubset(allowed):
            return False

        # skip tests by either only_type or platforms
        if only_type or platforms:
            for keys in get(self.config, "requirements.duts", dict()):
                # skip tests by only_type
                type_expr = "type" in self.config["requirements"]["duts"][keys] and \
                            self.config['requirements']['duts'][keys]["type"] == only_type
                if only_type and type_expr:
                    return True

                # skip test by platforms condition 1:
                plat_expr = "platform_name" in self.config['requirements']['duts'][keys] and \
                            self.config['requirements']['duts'][keys]["platform_name"] in platforms
                if platforms and plat_expr:
                    return True

        # no skip if neither only_type nor platforms is defined
        return False

    def get_tc_abspath(self, tc_file=None):
        """
        Get path to test case.

        :param tc_file: name of the file. If None, tcdir used instead.
        :return: absolute path.
        """
        if not tc_file:
            return os.path.abspath(self._args.tcdir)
        return os.path.abspath(tc_file)

    def set_config(self, config):
        """
        Set the configuration for this test case.

        :param config: dictionary
        :return: Nothing
        """
        self._config = config

    # Read Environment Configuration JSON file
    def _read_env_configs(self, env_cfg, iface):  # pylint: disable=no-self-use
        """
        Read environment configuration json file.

        :return: False if read fails, True otherwise.
        """
        data = None

        if env_cfg != '':
            env_cfg_filename = env_cfg
        else:
            env_cfg_filename = os.path.abspath(os.path.join(__file__,
                                                            os.path.pardir,
                                                            os.path.pardir,
                                                            os.path.pardir,
                                                            "env_cfg_json"))
        if os.path.exists(env_cfg_filename):
            with open(env_cfg_filename) as data_file:
                try:
                    data = json.load(data_file, object_pairs_hook=find_duplicate_keys)
                except ValueError as error:
                    self._logger.error(error)
                    raise InconclusiveError("Environment file {} read failed: {}".format(
                        env_cfg_filename, error))
        elif env_cfg != '':
            raise InconclusiveError('Environment file {} does not exist'.format(env_cfg))

        env = merge({}, data) if data else {}

        if iface:
            env = merge(env, {'sniffer': {'iface': iface}})
        else:
            env = merge(env, {'sniffer': {'iface': "Sniffer"}})
        return env

    # Read Execution Configuration file
    def _read_exec_configs(self, args):  # pylint: disable=too-many-branches
        """
        Read execution configuration file.

        :return: Nothing.
        :raises TestStepError if file cannot be read or merged into config, or if platform_name
        is not in allowed_platforms.
        """
        tc_cfg = None
        if args.tc_cfg:
            tc_cfg = args.tc_cfg
        # TODO: this bit is not compatible with IceteaManagement's --tc argument.
        elif isinstance(args.tc, string_types) and os.path.exists(args.tc + '.json'):
            tc_cfg = args.tc + '.json'
        if tc_cfg:
            if not os.path.exists(tc_cfg):
                self._logger.error("Execution configuration file {} does not exist.".format(tc_cfg))
                raise InconclusiveError(
                    "Execution configuration file {} does not exist.".format(tc_cfg))
            with open(tc_cfg) as data_file:
                try:
                    data = json.load(data_file, object_pairs_hook=find_duplicate_keys)
                    self._config = merge(self._config, data)
                except Exception as error:
                    self._logger.error("Testcase configuration read from file (%s) failed!", tc_cfg)
                    self._logger.error(error)
                    raise TestStepError("TC CFG read fail! {}".format(error))

        if args.type:
            self._config["requirements"]["duts"]["*"] = merge(
                self._config["requirements"]["duts"]["*"],
                {"type": args.type})

        if args.bin:
            self._config["requirements"]["duts"]["*"] = merge(
                self._config["requirements"]["duts"]["*"],
                {"application": {'bin': args.bin}})

        if args.platform_name:
            allowed = self._config["requirements"]["duts"]["*"].get("allowed_platforms")
            if allowed:
                if args.platform_name in allowed:
                    self._config["requirements"]["duts"]["*"][
                        "platform_name"] = args.platform_name
                else:
                    raise TestStepError("Required platform_name not in allowed_platforms.")
            else:
                self._config["requirements"]["duts"]["*"][
                    "platform_name"] = args.platform_name

    @staticmethod
    def _parse_config(**kwargs):
        """
        Internal helper for parsing configurations.

        :param kwargs: dict
        :return: dict
        """
        config = {
            "compatible": {
                "framework": {
                    "name": "Icetea",
                    "version": ">=1.0.0"
                },
                "automation": {
                    "value": True
                },
                "hw": {
                    "value": True
                }
            },
            "name": None,
            "type": None,
            "sub_type": None,
            "requirements": {
                "duts": {"*": {
                    "application": {
                        "bin": None
                    }
                }},
                "external": {
                    "apps": [
                    ]
                }
            }
        }
        integer_keys_found = False
        try:
            reqs = kwargs["requirements"]["duts"]
            if len(reqs) > 1:
                integer_keys_found = False
                new_keys = {}
                int_keys = []
                for key in reqs:
                    if isinstance(key, int):
                        integer_keys_found = True
                        int_keys.append(key)
                        val = reqs[key]
                        new_keys[str(key)] = val
                for key in new_keys:
                    reqs[key] = new_keys[key]
                for key in int_keys:
                    reqs.pop(key)

        except KeyError:
            pass

        for key in kwargs:
            if isinstance(kwargs[key], dict) and key in config:
                config[key] = merge(config[key], kwargs[key])
            else:
                config[key] = kwargs[key]

        return config, integer_keys_found



# pylint: disable=missing-docstring
# -*- coding: utf-8 -*-

"""
Copyright 2019 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import signal
import subprocess
import sys
import unittest
import platform
import os
import mock

from icetea_lib.IceteaManager import ExitCodes
from icetea_lib.tools.GenericProcess import GenericProcess
from icetea_lib.TestStepError import TestStepError


class MockLogger(object):
    def __init__(self):
        pass

    def info(self, *args, **kwargs):
        pass

    def debug(self, *args, **kwargs):
        pass

    def warning(self, *args, **kwargs):
        pass

    def error(self, *args, **kwargs):
        pass


class GenericProcessTestcase(unittest.TestCase):
    @unittest.skipIf(
        platform.system() == "Windows",
        "Launching process in Windows ends in icetea warning"
        "\"This OS is not supporting select.poll() or select.kqueue()\"")
    def test_quick_process(self):
        # Run testcase
        icetea_cmd = ["python", "icetea.py",
                      "--clean",
                      "--silent",
                      "--failure_return_value",
                      "--tc", "test_quick_process",
                      "--tcdir", '"{}"'.format(os.path.join("test", "tests"))]

        # Shouldn't need to join the argument array,
        # but it doesn't work for some reason without it (Python 2.7.12).
        retcode = subprocess.call(args=" ".join(icetea_cmd), shell=True)

        # Check success
        self.assertEqual(
            retcode, ExitCodes.EXIT_SUCCESS,
            "Non-success returncode {} returned.".format(retcode))

    def tearDown(self):
        # Run with --clean to clean up
        subprocess.call(
            "python icetea.py --clean ",
            shell=True)


@unittest.skipIf(sys.platform == 'win32', "process tests don't support Windows.")
class GenericProcessUnittests(unittest.TestCase):

    @mock.patch("icetea_lib.tools.GenericProcess.os")
    @mock.patch("icetea_lib.tools.GenericProcess.time")
    def test_stop_process(self, mocked_time, mocked_os):
        mocked_os.killpg = mock.MagicMock()
        mocked_time.sleep = mock.MagicMock(return_value=1)
        my_process = GenericProcess("test", logger=MockLogger())
        my_process.read_thread = mock.MagicMock()
        my_process.read_thread.stop = mock.MagicMock()
        my_process.stop_process()
        my_process.read_thread.stop.assert_called_once()
        mocked_proc = mock.MagicMock()
        pid = 11111111
        type(mocked_proc).pid = mock.PropertyMock(return_value=pid)
        mocked_proc.poll = mock.MagicMock()
        mocked_proc.poll.side_effect = [None, None, None, None, None,
                                        None, None, None, None, None,
                                        None, None, None, None, 0, 0, 0, 0, 0]
        type(my_process).proc = mock.PropertyMock(return_value=mocked_proc)
        my_process.stop_process()
        self.assertEqual(mocked_os.killpg.call_count, 3)
        mocked_os.killpg.assert_has_calls([mock.call(pid, signal.SIGINT),
                                           mock.call(pid, signal.SIGTERM),
                                           mock.call(pid, signal.SIGKILL)])

        my_process = GenericProcess("test", logger=MockLogger())
        my_process.read_thread = mock.MagicMock()
        my_process.read_thread.stop = mock.MagicMock()
        my_process.stop_process()
        my_process.read_thread.stop.assert_called_once()

        mocked_proc = mock.MagicMock()
        pid = 11111111
        type(mocked_proc).pid = mock.PropertyMock(return_value=pid)
        mocked_proc.poll = mock.MagicMock()
        mocked_proc.poll.side_effect = [None, None, None, None, None, None,
                                        None, None, None, 0, 0, 0]
        type(my_process).proc = mock.PropertyMock(return_value=mocked_proc)
        mocked_os.killpg.reset_mock()
        my_process.stop_process()
        self.assertEqual(mocked_os.killpg.call_count, 2)
        mocked_os.killpg.assert_has_calls([mock.call(pid, signal.SIGINT),
                                           mock.call(pid, signal.SIGTERM)])

    @mock.patch("icetea_lib.tools.GenericProcess.os", create=True)
    def test_stop_process_errors(self, mocked_os):
        mocked_os.killpg = mock.MagicMock(side_effect=OSError)
        my_process = GenericProcess("test", logger=MockLogger())

        mocked_proc = mock.MagicMock()
        pid = 11111111
        type(mocked_proc).pid = mock.PropertyMock(return_value=pid)
        type(my_process).proc = mock.PropertyMock(return_value=mocked_proc)
        my_process.stop_process()
        self.assertEqual(mocked_os.killpg.call_count, 3)

        mocked_os.killpg.reset_mock()

        mocked_os.killpg.side_effect = None
        mocked_os.killpg.return_value = 1
        mocked_proc.poll = mock.MagicMock()
        mocked_proc.poll.side_effect = [1, 1]
        with self.assertRaises(TestStepError):
            my_process.stop_process()

    def test_getters_and_setters(self):
        my_process = GenericProcess("test", logger=MockLogger())
        self.assertFalse(my_process.gdb)
        my_process.use_gdb()
        self.assertTrue(my_process.gdb)
        my_process.use_gdb(False)
        self.assertFalse(my_process.gdb)

        self.assertFalse(my_process.gdbs)
        my_process.use_gdbs()
        self.assertTrue(my_process.gdbs)
        self.assertEqual(my_process.gdbs_port, 2345)
        my_process.use_gdbs(False)
        self.assertFalse(my_process.gdbs)

        my_process.gdbs_port = 1234
        self.assertEqual(my_process.gdbs_port, 1234)

        self.assertFalse(my_process.vgdb)
        my_process.use_vgdb()
        self.assertTrue(my_process.vgdb)
        my_process.use_vgdb(False)
        self.assertFalse(my_process.vgdb)

        self.assertFalse(my_process.nobuf)
        my_process.no_std_buf()
        self.assertTrue(my_process.nobuf)
        my_process.no_std_buf(False)
        self.assertFalse(my_process.nobuf)

        self.assertIsNone(my_process.valgrind_xml)
        my_process.valgrind_xml = True
        self.assertTrue(my_process.valgrind_xml)

        self.assertFalse(my_process.valgrind_console)
        my_process.valgrind_console = True
        self.assertTrue(my_process.valgrind_console)

        self.assertFalse(my_process.valgrind_track_origins)
        my_process.valgrind_track_origins = True
        self.assertTrue(my_process.valgrind_track_origins)

        self.assertFalse(my_process.valgrind_extra_params)
        my_process.valgrind_extra_params = True
        self.assertTrue(my_process.valgrind_extra_params)

        self.assertFalse(my_process.ignore_return_code)
        my_process.ignore_return_code = True
        self.assertTrue(my_process.ignore_return_code)
        my_process.ignore_return_code = False
        self.assertFalse(my_process.ignore_return_code)

    def test_usevalgrind(self):
        my_process = GenericProcess("test", logger=MockLogger())
        with self.assertRaises(AttributeError):
            my_process.use_valgrind("test", True, True, True, True)
        my_process.use_valgrind("memcheck", 1, 2, 3, 4)
        self.assertEqual(my_process.valgrind, "memcheck")
        self.assertEqual(my_process.valgrind_xml, 1)
        self.assertEqual(my_process.valgrind_console, 2)
        self.assertEqual(my_process.valgrind_track_origins, 3)
        self.assertEqual(my_process.valgrind_extra_params, 4)


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import inspect
import os

import icetea_lib.tools.file.SessionFiles as files
from icetea_lib.Plugin.PluginBase import PluginBase


class FileApiPlugin(PluginBase):
    """
    Plugin interface for JsonFile plugin.
    """
    def __init__(self):
        super(FileApiPlugin, self).__init__()  # pylint: disable=useless-super-delegation
        self.bench = None

    def init(self, bench=None):
        """
        Init function to store the Bench object reference.

        :param bench: Bench
        :return: Nothing
        :raises AttributeError if bench is None.
        """
        self.bench = bench
        if self.bench is None:
            raise AttributeError("Bench instance not present!")

    def get_bench_api(self):
        """
        Get the descriptor for the plugin interface.

        :return: dict
        """
        return {"JsonFile": self._jsonfileconstructor}

    def _jsonfileconstructor(self, filename=None, filepath=None, logger=None):
        """
        Constructor method for the JsonFile object.

        :param filename: Name of the file
        :param filepath: Path to the file
        :param logger: Optional logger.
        :return: JsonFile
        """
        if filepath:
            path = filepath
        else:
            tc_path = os.path.abspath(os.path.join(inspect.getfile(self.bench.__class__),
                                                   os.pardir))
            path = os.path.abspath(os.path.join(tc_path, os.pardir, "session_data"))
        name = "default_file.json" if not filename else filename
        log = self.bench.logger if not logger else logger
        self.bench.logger.info("Setting json file location to: {}".format(path))
        return files.JsonFile(log, path, name)



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
# pylint: disable=invalid-name,missing-docstring,too-many-branches,len-as-condition,unused-argument
# pylint: disable=unused-variable,redefined-builtin,attribute-defined-outside-init
# pylint: disable=pointless-string-statement
import re
from datetime import datetime
from threading import Lock, Thread

import pyshark
import icetea_lib.LogManager as LogManager
from icetea_lib.TestStepError import TestStepError


class NwPacket(object):

    @staticmethod
    def verify(packet, expected_values):
        lines = packet.packetStr.splitlines()
        field_values = dict()  # field keys assumed to be unique accross layers
        for layer_key, layer_value in expected_values.items():
            # support for legacy JSON interface
            if re.match(r"^.+\..+", layer_key):
                layer_key, field_key = layer_key.split('.')
                layer_value = {field_key: layer_value}
            layer_found = False
            for line in lines:
                if not layer_found:
                    match = re.search(r"^Layer "+layer_key+":", line)
                    if not match:
                        continue
                    layer_found = True
                for field_key, field_value in layer_value.items():
                    if field_key in field_values and field_values[field_key]:
                        continue
                    else:
                        field_values[field_key] = False
                    # Whole row match (key starts with '*')
                    if field_key.startswith("*"):
                        match = re.search(r"(\t)" + field_value, line)
                        if match:
                            field_values[field_key] = True
                        continue
                    match = re.search(r"(\t|(, )|= )" + field_key + r":[\s]{0,}([^,]*)[,]{0,1}",
                                      line)
                    if not match:
                        continue
                    if re.search(field_value, match.group(3)):
                        field_values[field_key] = True
                    else:
                        continue
            if not layer_found:
                return False
            for field_key, field_value in field_values.items():
                if field_value is False:
                    return False
        return True

    def __init__(self, packet, mark=None):
        self.__marks = []
        self.timestamp = datetime.now()
        self.packetStr = str(packet)
        self.packet = packet
        self.appendMark(mark)

    def isMark(self, mark):
        return mark in self.__marks

    def appendMark(self, mark):
        if mark:
            self.__marks.append(mark)

    def __str__(self):
        return str(self.packet)

    def getMarks(self):
        return ','.join(self.__marks)


class NwPacketManager(object):

    def __init__(self):
        self.__packets = []
        self.__marks = []
        self.__lock = Lock()
        self.setMarkForHead('start')
        self.logger = LogManager.get_bench_logger("bench", "WS")

    def setMarkForHead(self, mark):
        if self.hasPackets():
            # set mark
            self.__packets[-1].appendMark(mark)
            return

        self.__lock.acquire()
        self.__marks.append(mark)
        self.__lock.release()

    def push(self, packet, mark=None):
        packet = NwPacket(packet)

        self.__lock.acquire()
        while len(self.__marks) > 0:
            packet.appendMark(self.__marks.pop())
        self.__lock.release()

        self.__packets.append(packet)
        # print("Got packet, count now: %i" % len(self))

    def count(self):
        return len(self.__packets)

    def getPackets(self):
        return self.__packets

    def lastPacketIndex(self):
        return len(self.__packets) - 1

    def hasPackets(self):
        return len(self.__packets) > 0

    def __getitem__(self, index):
        return self.__packets[index].packet

    ''' print packet to console between markers.
    By default it prints all packets from beginning to end.
    usage example:
    self.wshark.setMark("S1")
    self.delay(10)
    self.wshark.setMark("S2")
    self.wshark.printPackets("S1", "S2")
    '''
    def printPackets(self, startMarker='start', endMarker=None):
        if not self.hasPackets():
            raise ValueError("Not packets available")

        start_index = self.findIndexByMark(startMarker)
        end_index = self.lastPacketIndex() if endMarker is None else self.findIndexByMark(endMarker)
        for n in range(start_index, end_index):
            print(self[n])

    # verify packet(s) that it contains valid contents
    # Usage example:
    #   verifyPackets( [{"WPAN": {"Command Identifier": "Beacon Request"}}] )
    #     OR  verifyPackets( [{"WPAN.Command Identifier": "Beacon Request"}] )
    # raise TestStepError exception if packet(s) not contains expected content
    def verifyPackets(self, expectedPackets, startMarker='start', endMarker=None):

        if not self.hasPackets():
            raise ValueError("Not packets available")

        start_index = self.findIndexByMark(startMarker)
        end_index = self.lastPacketIndex() if endMarker is None else self.findIndexByMark(endMarker)

        is_ok, expected_that_not_found, packet = self.__verify_packets(expectedPackets,
                                                                       start_index,
                                                                       end_index)
        if not is_ok:
            # @todo print why packet didn't match to expected
            raise TestStepError("Packet not found: " + str(expected_that_not_found))

        self.logger.debug("verifyPackets success")

    # Count number of packets that match the given specifications (packet fields)
    # Usage example:
    #   countPackets( {"IPV6":{"Destination":"fe80::166e:a00:0:2"}} )
    def countPackets(self, expectedPacket, startMarker='start', endMarker=None):

        if not self.hasPackets():
            raise ValueError("Not packets available")

        start_index = self.findIndexByMark(startMarker)
        end_index = self.lastPacketIndex() if endMarker is None else self.findIndexByMark(endMarker)

        count = self.__count_packets(expectedPacket, start_index, end_index)
        return count

    def findIndexByMark(self, mark):
        index = 0
        for pck in self.__packets:
            if pck.isMark(mark):
                return index
            index += 1
        return None

    def FindNext(self, expectedPacket, begIndex, toIndex):
        for index in range(begIndex, toIndex+1):
            is_ok = NwPacket.verify(self.__packets[index], expectedPacket)
            if is_ok:
                return True, index, self.__packets[index]
        raise LookupError("Not found")

    def __verify_packets(self, expectedPackets, startIndex, endIndex):
        position = startIndex
        for expectedContent in expectedPackets:
            try:
                is_ok, position, match = self.FindNext(expectedContent, position, endIndex)
                if not is_ok:
                    return False, expectedContent, match
                position = position + 1
            except LookupError as msg:
                # Not found
                # print("Not Found: %s" % msg)
                return False, expectedContent, None
        return True, None, None

    def __count_packets(self, expectedContent, startIndex, endIndex):
        position = startIndex
        count = 0
        is_ok = True
        try:
            while is_ok:
                is_ok, position, match = self.FindNext(expectedContent, position, endIndex)
                if is_ok:
                    count = count + 1
                position = position + 1
        except LookupError:
            return count
        return count


class Wireshark(NwPacketManager):

    __iface = 'Sniffer'
    __captureThreadFile = None

    fileLoggingCapture = None
    liveCapture = None

    def __init__(self):
        NwPacketManager.__init__(self)
        self.logger = LogManager.get_bench_logger("bench", "WS")
        self.__captureThreadLive = None
        self.__captureThreadFile = None

    def setMark(self, mark):
        self.setMarkForHead(mark)

    # Start live network capturing to a file
    def startCapture(self, iface, file, tshark_arguments=None):
        self.__iface = iface
        if file:
            # first capture data to a file
            self.logger.debug('Start wireshark capturing to a file: %s', file)
            kwargs = dict(iface=self.__iface, file=file)
            if isinstance(tshark_arguments, dict):
                for key in tshark_arguments.keys():
                    kwargs[key] = tshark_arguments[key]
            self.__captureThreadFile = Thread(target=self.__sniff_to_file, kwargs=kwargs)
            self.__captureThreadFile.setName("NW file capture")
            self.__captureThreadFile.start()

        # capture live data to a python side..
        kwargs = dict(iface=self.__iface)
        if isinstance(tshark_arguments, dict):
            for key in tshark_arguments.keys():
                kwargs[key] = tshark_arguments[key]
        self.__captureThreadLive = Thread(target=self.__live_capture, kwargs=kwargs)
        self.__captureThreadLive.setName("NW live capture")
        self.__captureThreadLive.start()

    # Load captured packets from a file
    def loadCapture(self, file):
        self.logger.debug('Loading capture file: %s', file)
        capture = pyshark.FileCapture(input_file=file)
        if capture is None:
            raise ValueError('Loading capture file FAILED.')
        for packet in capture:
            self.push(packet)
        self.logger.debug('Done loading capture file.')

    # Stop live network data logging
    def stopCapture(self):
        if self.fileLoggingCapture:
            self.fileLoggingCapture.close()
            self.__captureThreadFile.join(timeout=5)
        self.liveLoggingCapture.close()
        self.__captureThreadLive.join(timeout=5)
        return self.count()

    # Get captured packets
    def getCaptures(self):
        return self.getPackets()

    # Once pyshark supports it, add preference overriding arguments
    def __live_capture(self, iface, decode_as=None):
        self.logger.debug("Sniffing if: '%s' -> live" % iface)
        self.liveLoggingCapture = pyshark.LiveCapture(interface=iface, decode_as=decode_as)
        for packet in self.liveLoggingCapture.sniff_continuously():
            self.push(packet)
        self.logger.debug("live sniffing ends")

    # Once pyshark supports it, add preference overriding arguments
    def __sniff_to_file(self, iface, file, decode_as=None):
        self.logger.debug("Sniffing if: '%s' -> file: %s" % (iface, file))
        self.fileLoggingCapture = pyshark.LiveCapture(interface=iface, output_file=file,
                                                      decode_as=decode_as)
        ret = self.fileLoggingCapture.sniff()  # pylint: disable=unused-variable
        self.logger.debug("file sniffing ends")



# pylint: disable=missing-docstring,no-self-use,protected-access,invalid-name,too-few-public-methods
# pylint: disable=wrong-import-order
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import os
import subprocess
import sys
import unittest

from icetea_lib.TestBench.Configurations import Configurations
from icetea_lib.TestStepError import InconclusiveError, TestStepError
from icetea_lib.IceteaManager import ExitCodes

from test.dummy_dut import compile_dummy_dut


class MockArgs(object):
    def __init__(self):
        self.tc_cfg = None
        self.channel = None
        self.type = None
        self.bin = None
        self.platform_name = None


class ConfigMixerTests(unittest.TestCase):

    def test_read_env_config(self):
        cmixer = Configurations()

        with self.assertRaises(InconclusiveError):
            env_cfg = cmixer._read_env_configs("test_config", "test_iface")

        env_cfg = cmixer._read_env_configs(
            os.path.abspath(os.path.join(__file__, "..", "data", "test_env_cfg.json")),
            "test_iface")
        self.assertDictEqual(env_cfg, {"test_config": "test",
                                       "sniffer": {"iface": "test_iface"}})

        with self.assertRaises(InconclusiveError):
            env_cfg = cmixer._read_env_configs(
                os.path.abspath(os.path.join(__file__, "..", "data",
                                             "test_env_cfg_with_duplicates.json")), "test_iface")

    def test_read_exec_config(self):
        cmixer = Configurations()
        args = MockArgs()
        args.tc_cfg = "non_existent_file.json"
        with self.assertRaises(InconclusiveError):
            cmixer._read_exec_configs(args)
        args.tc_cfg = os.path.abspath(os.path.join(__file__, "..", "data", "test_env_cfg.json"))
        args.channel = "1"
        args.type = "type"
        args.bin = "bin"
        args.platform_name = "platform"
        cmixer._read_exec_configs(args)
        self.assertDictContainsSubset(
            {"test_config": "test",
             "requirements": {
                 "duts": {
                     "*": {
                         "type": "type",
                         "application": {
                             "bin": "bin"
                         },
                         "platform_name": "platform"
                     }
                 },
                 "external": {
                     "apps": []
                 }
             }
            },
            cmixer.config
        )

        args.tc_cfg = os.path.abspath(os.path.join(__file__, "..", "data",
                                                   "test_env_cfg_with_duplicates.json"))
        with self.assertRaises(TestStepError):
            cmixer._read_exec_configs(args)

        args.tc_cfg = os.path.abspath(os.path.join(__file__, "..", "data", "test_env_cfg.json"))

        args.platform_name = "not-in-allowed"
        cmixer.config["requirements"]["duts"]["*"]["allowed_platforms"] = ["allowed"]
        with self.assertRaises(TestStepError):
            cmixer._read_exec_configs(args)
        args.platform_name = "allowed"
        cmixer._read_exec_configs(args)
        self.assertDictContainsSubset(
            {"test_config": "test",
             "requirements": {
                 "duts": {
                     "*": {
                         "type": "type",
                         "application": {
                             "bin": "bin"
                         },
                         "allowed_platforms": ["allowed"],
                         "platform_name": "allowed"
                     }
                 },
                 "external": {
                     "apps": []
                 }
             }
            },
            cmixer.config
        )

    @unittest.skipIf(sys.platform == 'win32', "windows does't support process tests")
    def test_config_parse_corner_case_33(self):
        compile_dummy_dut()
        script_file = os.path.abspath(os.path.join(__file__, os.path.pardir, os.path.pardir,
                                                   "icetea.py"))
        tcdir = os.path.abspath(os.path.join(__file__, os.path.pardir))

        retcode = subprocess.call(
            "python " + script_file + " --clean -s "
                                      "--tc test_config_parse_corner_case --failure_return_value "
                                      "--tcdir " + tcdir,
            shell=True)
        self.assertEqual(retcode, ExitCodes.EXIT_SUCCESS)

    def test_parse_config(self):
        basic_config = {
            "name": "my_test",
            "type": "regression",
            "sub_type": None,
            "requirements": {
                "duts": {"*": {
                    "application": {
                        "bin": None
                    }
                }}
            }
        }
        expected = {
            "compatible": {
                "framework": {
                    "name": "Icetea",
                    "version": ">=1.0.0"
                },
                "automation": {
                    "value": True
                },
                "hw": {
                    "value": True
                }
            },
            "name": "my_test",
            "type": "regression",
            "sub_type": None,
            "requirements": {
                "duts": {"*": {
                    "application": {
                        "bin": None
                    }
                }},
                "external": {
                    "apps": [
                    ]
                }
            }
        }
        retval, int_keys = Configurations._parse_config(**basic_config)
        self.assertDictEqual(retval, expected)
        self.assertFalse(int_keys)

        basic_config = {
            "name": "my_test",
            "type": "regression",
            "sub_type": None,
            "requirements": {
                "duts": {
                    "*": {
                        "application": {
                            "bin": None
                        }
                    },
                    1: {"nick": "intkey"}}
            }
        }
        expected = {
            "compatible": {
                "framework": {
                    "name": "Icetea",
                    "version": ">=1.0.0"
                },
                "automation": {
                    "value": True
                },
                "hw": {
                    "value": True
                }
            },
            "name": "my_test",
            "type": "regression",
            "sub_type": None,
            "requirements": {
                "duts": {
                    "*": {
                        "application": {
                            "bin": None
                        }
                    },
                    "1": {"nick": "intkey"}},
                "external": {
                    "apps": [
                    ]
                }
            }
        }

        retval, int_keys = Configurations._parse_config(**basic_config)
        self.assertDictEqual(retval, expected)
        self.assertTrue(int_keys)



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Randomize class for generating randomized content.
"""

import random
import string
from inspect import isfunction

from icetea_lib.Randomize.seed import SeedInteger, SeedString, SeedStringArray


class Randomize(object):
    """
    Randomize class, collection of static methods for generating randomized content.
    """
    @staticmethod
    def random_integer(max_value, min_value=0):
        """
        :param max_value: Maximum value, int
        :param min_value: Minimum value, int, default is 0
        :return: SeedInteger
        """
        return SeedInteger(random.randint(min_value, max_value))

    @staticmethod
    def random_list_elem(str_list):
        """
        :param str_list: a pre-defined string list
        :return: SeedString()
        """
        if isinstance(str_list, list):
            for elem in str_list:
                if not isinstance(elem, str):
                    raise TypeError("list element can only be string")
            return SeedString(random.choice(str_list))
        return None

    @staticmethod
    def random_string(max_len=1, min_len=1, chars=string.ascii_letters, **kwargs):
        """
        :param max_len: max value of len(string)
        :param min_len: min value of len(string)
        :param chars: can be sting, list of strings or function pointer.
        Randomly choose one if given a list of strings
        :param kwargs: keyword arguments for chars if it's function pointer
        :return: SeedString()
        """
        if isinstance(chars, list):
            # assume each element is a str
            chars = ''.join(chars)

        if isinstance(chars, str):
            return SeedString(
                ''.join(random.choice(chars) for _ in range(random.randint(min_len, max_len))))
        elif isfunction(chars):
            # this function is assumed to return/generate one character each time it is called
            return SeedString(
                ''.join(chars(**kwargs) for _ in range(random.randint(min_len, max_len))))
        else:
            raise ValueError("chars should be string, list, or function pointer")

    @staticmethod
    def random_array_elem(str_array):
        """
        :param str_array:a pre-defined string array
        :return: SeedStringArray()
        """
        return SeedStringArray([str(Randomize.random_list_elem(str_array))])

    @staticmethod
    def random_string_array(max_len=1, min_len=1,
                            elem_max_len=1, elem_min_len=1,
                            strings=string.ascii_letters, **kwargs):
        """
        :param max_len: max value of len(array)
        :param min_len: min value of len(array)
        :param elem_max_len: max value of len(array[index])
        :param elem_min_len: min value of len(array[index])
        :param strings: allowed string characters in each element of array,
        or predefined list of strings, or function pointer
        :param **kwargs: keyworded arguments for strings if it's a function pointer
        :return: SeedStringArray
        """
        string_array = list()
        for _ in range(random.randint(min_len, max_len)):
            string_array.append(Randomize.random_string(max_len=elem_max_len, min_len=elem_min_len,
                                                        chars=strings, **kwargs).value)

        return SeedStringArray(string_array)



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

DutDetection module. Uses mbedls to detect available devices for LocalAllocator.
"""

# Disable "Method could be function" warning
# pylint: disable=R0201

from serial.tools import list_ports
from serial import SerialException, Serial

from icetea_lib.ResourceProvider.Allocators.exceptions import AllocationError


class DutDetection(object):
    """
    DutDetection class. Contains methods to detect usable ports and available devices using
    mbedls and serial module.
    """

    def __init__(self):
        self.mbeds = None
        try:
            from mbed_lstools.main import create
            self.mbeds = create()
        except ImportError:
            raise AllocationError("mbedls is missing")

    @staticmethod
    def is_port_usable(port_name):
        """
        Static is_port_usable method. Tries to create instance of Serial object. to confirm that
        the port is usable.

        :param port_name: Name of port
        :return: True or False
        """
        try:
            #Disable "unused variable" warning
            #pylint: disable=W0612
            ser = Serial(port=port_name)
            return True
        except SerialException:
            return False

    def get_available_devices(self):
        """
        Gets available devices using mbedls and self.available_edbg_ports.

        :return: List of connected devices as dictionaries.
        """
        connected_devices = self.mbeds.list_mbeds() if self.mbeds else []
        # Check non mbedOS supported devices.
        # Just for backward compatible reason - is obsolete..
        edbg_ports = self.available_edbg_ports()
        for port in edbg_ports:
            connected_devices.append({
                "platform_name": "SAM4E",
                "serial_port": port,
                "mount_point": None,
                "target_id": None,
                "baud_rate": 460800
            })
        for dev in connected_devices:
            dev['state'] = "unknown"

        return connected_devices

    def available_edbg_ports(self):
        """
        Finds available EDBG COM ports.

        :return: list of available ports
        """
        ports_available = sorted(list(list_ports.comports()))
        edbg_ports = []
        for iport in ports_available:
            port = iport[0]
            desc = iport[1]
            hwid = iport[2]
            if str(desc).startswith("EDBG Virtual COM Port") or \
                            "VID:PID=03EB:2111" in str(hwid).upper():
                # print("%-10s: %s (%s)\n" % (port, desc, hwid))
                try:
                    edbg_ports.index(port, 0)
                    print("There is multiple %s ports with same number!" % port)
                except ValueError:
                    edbg_ports.append(port)
        # print("Detected %i DUT's" % len(edbg_ports))
        return edbg_ports



# pylint: disable=missing-docstring,expression-not-assigned,no-self-use,unused-variable,
# pylint: disable=unused-argument,protected-access

import logging

import os
import unittest
import mock

from jsonschema import ValidationError

import icetea_lib.LogManager as LogManager
from icetea_lib.LogManager import ContextFilter, traverse_json_obj, get_external_logger
from icetea_lib.tools.tools import IS_PYTHON3


class FunctionTests(unittest.TestCase):
    def setUp(self):
        pass

    @mock.patch("icetea_lib.LogManager.os.path.join")
    def test_verbosity_set_to_warn(self, mock_os):
        LogManager.VERBOSE_LEVEL = 0
        LogManager.SILENT_ON = False
        logger = get_external_logger("test_logger", "TST", False)
        logger = logging.getLogger("test_logger")
        self.assertTrue(logger.handlers[0].level == logging.WARNING)

    @mock.patch("icetea_lib.LogManager.os.path.join")
    def test_verbosity_set_to_info(self, mock_os):
        LogManager.VERBOSE_LEVEL = 1
        LogManager.SILENT_ON = False
        logger = get_external_logger("test_logger2", "TST", False)
        logger = logging.getLogger("test_logger2")
        self.assertTrue(logger.handlers[0].level == logging.INFO)

    @mock.patch("icetea_lib.LogManager.os.path.join")
    def test_verbosity_set_to_debug(self, mock_os):
        LogManager.VERBOSE_LEVEL = 2
        LogManager.SILENT_ON = False
        logger = get_external_logger("test_logger3", "TST", False)
        logger = logging.getLogger("test_logger3")
        self.assertTrue(logger.handlers[0].level == logging.DEBUG)

    @mock.patch("icetea_lib.LogManager.os.path.join")
    def test_verbosity_set_to_silent(self, mock_os):
        LogManager.VERBOSE_LEVEL = 2
        LogManager.SILENT_ON = True
        logger = get_external_logger("test_logger4", "TST", False)
        logger = logging.getLogger("test_logger4")
        self.assertTrue(logger.handlers[0].level == logging.ERROR)


class ConfigFileTests(unittest.TestCase):
    def setUp(self):
        self.original_config = LogManager.LOGGING_CONFIG

    def test_configs_read(self):
        filepath = os.path.abspath(os.path.join(__file__, os.path.pardir, "tests",
                                                "does_not_exist.json"))
        with self.assertRaises(IOError):
            LogManager.init_base_logging(config_location=filepath)
        filepath = os.path.abspath(os.path.join(__file__, os.path.pardir, "tests",
                                                "logging_config.json"))
        LogManager._read_config(filepath)

    def test_configs_merge(self):
        filepath = os.path.abspath(os.path.join(__file__, os.path.pardir, "tests",
                                                "logging_config.json"))
        LogManager._read_config(filepath)
        self.assertDictEqual(LogManager.LOGGING_CONFIG.get("test_logger"), {"level": "ERROR"})

    def test_configs_schema_validation(self):
        filepath = os.path.abspath(os.path.join(__file__, os.path.pardir, "tests",
                                                "erroneous_logging_config.json"))
        with self.assertRaises(ValidationError):
            LogManager._read_config(filepath)
        filepath = os.path.abspath(os.path.join(__file__, os.path.pardir, "tests",
                                                "logging_config.json"))
        LogManager._read_config(filepath)

    def tearDown(self):
        LogManager.LOGGING_CONFIG = self.original_config


class ContextFilterTest(unittest.TestCase):
    def setUp(self):
        self.contextfilter = ContextFilter()

    def test_filter_base64(self):
        msg = "aaa="
        record = create_log_record(msg)
        self.contextfilter.filter(record)
        self.assertEqual(msg, record.msg)

        msg = []
        [msg.append("a") for _ in range(ContextFilter.MAXIMUM_LENGTH)]
        msg = "".join(msg)
        record = create_log_record(msg)
        self.contextfilter.filter(record)
        expected = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa...[9950 more bytes]"
        self.assertEqual(expected, record.msg)

        msg = []
        [msg.append("a") for _ in range(ContextFilter.MAXIMUM_LENGTH * 2)]
        msg = "".join(msg)
        record = create_log_record(msg)
        self.contextfilter.filter(record)
        expected = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa...[19950 more bytes]"
        self.assertEqual(expected, record.msg)

        msg = []
        [msg.append("a") for _ in range(ContextFilter.MAXIMUM_LENGTH - 1)]
        msg = "".join(msg)
        record = create_log_record(msg)
        self.contextfilter.filter(record)
        self.assertEqual(msg, record.msg)

        msg = []
        [msg.append("a") for _ in range(ContextFilter.MAXIMUM_LENGTH + 1)]
        msg = "".join(msg)
        record = create_log_record(msg)
        self.contextfilter.filter(record)
        expected = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa...[9951 more bytes]"
        self.assertEqual(expected, record.msg)

    def test_filter_human_readable(self):
        msg = [" "]
        [msg.append("a") for _ in range(ContextFilter.MAXIMUM_LENGTH)]
        msg = "".join(msg)
        record = create_log_record(msg)
        expected = " aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa...[9951 more bytes]"
        self.contextfilter.filter(record)
        self.assertEqual(expected, record.msg)

    def test_filter_binary_data(self):
        msg = []
        [msg.append(os.urandom(1024)) for _ in range(ContextFilter.MAXIMUM_LENGTH +1)]
        msg = b"".join(msg)
        expected = "{}...[10240974 more bytes]".format(msg[:50])
        record = create_log_record(msg)
        self.contextfilter.filter(record)

        self.assertEqual(expected, record.msg)

        msg = []
        [msg.append(u'\ufffd') for _ in range(ContextFilter.MAXIMUM_LENGTH + 1)]
        msg = "".join(msg)
        if not IS_PYTHON3:
            expected = u"{}...[9951 more bytes]".format(repr(msg[:50]))
        else:
            expected = u"{}...[9951 more bytes]".format(msg[:50])

        record = create_log_record(msg)
        self.contextfilter.filter(record)
        self.assertEqual(expected, record.msg)


class TraverseJsonObjTest(unittest.TestCase):
    def test_stays_untouched(self):
        test_dict = [{"a": "aa", "b": ["c", "d", {"e": "aa", "f": "aa"}]}]

        self.assertEqual(test_dict, traverse_json_obj(test_dict))

    def test_all_modified(self):
        test_dict = [{"a": "aa", "b": ["c", "d", {"e": "aa", "f": "aa"}]}]

        def modify(value):
            return "bb" if value == "aa" else value

        expected = [{"a": "bb", "b": ["c", "d", {"e": "bb", "f": "bb"}]}]
        self.assertEqual(expected, traverse_json_obj(test_dict, callback=modify))


class CustomFormatterTest(unittest.TestCase):

    def test_benchformatter_formats_timestamp(self):  # pylint: disable=invalid-name
        formatter = LogManager.BenchFormatter("%(asctime)s | %(source)s %(type)s %(threadName)s: "
                                              "%(message)s", "%Y-%m-%dT%H:%M:%S.%FZ")

        record = create_log_record("test_message")
        time_str = formatter.formatTime(record, "%Y-%m-%dT%H:%M:%S.%FZ")

        # Check that T exists, should be in between date and time.
        self.assertTrue(time_str.rfind("T") > 0)

        # Check that date format matches ISO-8601
        date = time_str[:time_str.rfind("T")]
        self.assertRegexpMatches(date, r"^([0-9]{4})(-?)(1[0-2]|0[1-9])\2(3[01]|0[1-9]|[12][0-9])$")

        # Chech that time format matches ISO-8601
        time_ending = time_str[time_str.rfind("T") + 1: time_str.rfind(".")]
        self.assertRegexpMatches(time_ending, r"^(2[0-3]|[01][0-9]):?([0-5][0-9]):?([0-5][0-9])$")

        # Check that time_str ends with Z to show it's UTC
        self.assertTrue(time_str.endswith("Z"))

        # Check that milliseconds exist
        millis = time_str[time_str.rfind(".") + 1: time_str.rfind("Z")]
        self.assertRegexpMatches(millis, r"([0-9][0-9][0-9])")


# Helper function
def create_log_record(msg):
    return logging.LogRecord(name="", level=logging.ERROR, pathname="",
                             lineno=0, msg=msg, args=None, exc_info=None,
                             func=None)



# pylint: disable=too-many-lines
# pylint: disable=too-many-public-methods,too-many-instance-attributes,too-many-arguments
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

BenchApi module.
"""

from six import iteritems

from icetea_lib.tools.deprecated import deprecated
from icetea_lib.TestBench.Configurations import Configurations
from icetea_lib.TestBench.ArgsHandler import ArgsHandler
from icetea_lib.TestBench.Logger import Logger
from icetea_lib.TestBench.Resources import ResourceFunctions
from icetea_lib.TestBench.Plugins import Plugins
from icetea_lib.TestBench.Commands import Commands
from icetea_lib.TestBench.Results import Results
from icetea_lib.TestBench.NetworkSniffer import NetworkSniffer
from icetea_lib.TestBench.BenchFunctions import BenchFunctions


class BenchApi(object):
    """
    BenchApi class. Implements methods that provide access to the implementations contained in
    the other TestBench modules.
    """
    def __init__(self, **kwargs):
        super(BenchApi, self).__init__()
        self._arguments = ArgsHandler()
        self._logger = Logger()
        self._configurations = Configurations(args=self._arguments.args,
                                              logger=self.logger, **kwargs)

        self._resources = ResourceFunctions(self._arguments.args, self.logger, self._configurations)
        self._plugins = Plugins(self.logger, self.env, self._arguments.args,
                                self._configurations.config)
        self._resultfunctions = Results(self.logger, self._resources,
                                        self._configurations, self._arguments.args)
        self._benchfunctions = BenchFunctions(self.resource_configuration,
                                              self._resources, self._configurations)
        self._commands = Commands(self.logger, self._plugins, self._resources, self._arguments.args,
                                  self._benchfunctions)
        self._nwsniffer = NetworkSniffer(self._resources,
                                         self._configurations, self._arguments.args,
                                         self.logger)

    def _init(self):
        """
        Initialize internal class instances.
        """
        self._logger.init_logger(self._configurations.test_name,
                                 self.args.verbose,
                                 self.args.silent,
                                 self.args.color,
                                 self.args.disable_log_truncate)
        self._benchfunctions.init(logger=self.logger)
        self._configurations.init(self.logger)
        self._resources.init(self._commands, self.logger)
        self._plugins.init(self, self.logger)
        self._resultfunctions.init(self.logger)
        self._nwsniffer.init(self.logger)
        self._commands.init(self.logger)
        self.__wrap_obsoleted_functions()

    def get_logger(self):
        """
        Get logger function. Calls Logger.get_logger().

        :return: BenchLoggerAdapter
        """
        return self._logger.get_logger()

    @property
    def args(self):
        """
        Gets known args from ArgsHandler.

        :return: Namespace
        """
        return self._arguments.args

    @args.setter
    def args(self, value):
        """
        Setter for known args.
        """
        self._arguments.args = value

    @property
    def unknown(self):
        """
        Gets unknown args from ArgsHandler.
        """
        return self._arguments.unknown

    @unknown.setter
    def unknown(self, value):
        """
        Setter for unknown args.
        """
        self._arguments.unknown = value

    @property
    def logger(self):
        """
        :return: Logger from Logger. BenchLoggerAdapter usually.
        """
        return self._logger.get_logger()

    @logger.setter
    def logger(self, value):
        """
        Sets logger.
        """
        self._logger.set_logger(value)

    @property
    def test_name(self):
        """
        Returns test_name from Configurations.
        """
        return self._configurations.test_name

    @property
    def config(self):
        """
        :return: Configurations.config
        """
        return self._configurations.config

    @config.setter
    def config(self, value):
        """
        Sets Configurations.config.
        """
        self._configurations.config = value

    @property
    def env(self):
        """
        :return: Configurations.env
        """
        return self._configurations.env

    def sync_cli(self, dut, generator_function=None, generator_function_args=None, retries=None,
                 command_timeout=None):
        """
        Synchronize cli for a dut using custom function.

        :param dut: Dut
        :param generator_function: callable
        :param generator_function_args: list of arguments for generator_function
        :param retries: int, if set to 0 will skip command entirely (for unit testing purposes)
        :param command_timeout: int
        :raises: TestStepError: if synchronization fails.
        :raises: AttributeError: if retries is set to 0, unit testing reasons.
        """
        return self._commands.sync_cli(dut, generator_function, generator_function_args,
                                       retries, command_timeout)

    def is_hardware_in_use(self):
        """
        :return: True if type is hardware
        """
        return self._configurations.is_hardware_in_use()

    def get_platforms(self):
        """
        Get list of dut platforms.

        :return: list
        """
        return self._resources.get_platforms()

    def get_serialnumbers(self):
        """
        Get list of dut serial numbers.

        :return: list
        """
        return self._resources.get_serialnumbers()

    def get_test_component(self):
        """
        Get test component.

        :return: string
        """
        return self._configurations.get_test_component()

    def get_features_under_test(self):
        """
        Get features tested by this test case.

        :return: list
        """
        return self._configurations.get_features_under_test()

    def get_allowed_platforms(self):
        """
        Return list of allowed platfroms from requirements.

        :return: list
        """
        return self._configurations.get_allowed_platforms()

    def status(self):
        """
        Get TC implementation status.

        :return: string or None
        """
        return self._configurations.status()

    def type(self):
        """
        Get test case type.

        :return: string or None
        """
        return self._configurations.type()

    def subtype(self):
        """
        Get test case subtype.

        :return: string or None
        """
        return self._configurations.subtype()

    def get_config(self):
        """
        Get test case configuration.

        :return: dict
        """
        return self._configurations.get_config()

    def skip(self):
        """
        Get skip value.

        :return: Boolean or None
        """
        return self._configurations.skip()

    def skip_info(self):
        """
        Get the entire skip dictionary.

        :return: dictionary or None
        """
        return self._configurations.skip_info()

    def skip_reason(self):
        """
        Get skip reason.

        :return: string
        """
        return self._configurations.skip_reason()

    def get_tc_abspath(self, tc_file=None):
        """
        Get path to test case.

        :param tc_file: name of the file. If None, tcdir used instead.
        :return: absolute path.
        """
        return self._configurations.get_tc_abspath(tc_file)

    def set_config(self, config):
        """
        Set the configuration for this test case.

        :param config: dictionary
        :return: Nothing
        """
        return self._configurations.set_config(config)

    def check_skip(self):
        """
        Check if tc should be skipped

        :return: Boolean
        """
        return self._configurations.check_skip()

    def load_plugins(self):
        """
        Initialize PluginManager and Load bench related plugins.

        :return: Nothing
        """
        return self._plugins.load_plugins()

    def init_duts(self):
        """
        Initialize Duts, and the network sniffer.

        :return: Nothing
        """
        return self._resources.init_duts(self)

    def duts_release(self):
        """
        Release Duts.

        :return: Nothing
        """
        return self._resources.duts_release()

    @property
    def resource_configuration(self):
        """
        Getter for __resource_configuration.

        :return: ResourceConfig
        """
        return self._resources.resource_configuration

    @resource_configuration.setter
    def resource_configuration(self, value):
        """
        Setter for __resource_configuration.

        :param value: ResourceConfig
        :return: Nothing
        """
        self._resources.resource_configuration = value

    def dut_count(self):
        """
        Getter for dut count from resource configuration.

        :return: int
        """
        return self._resources.dut_count()

    def get_dut_count(self):
        """
        Get dut count.

        :return: int
        """
        return self._resources.get_dut_count()

    @property
    def resource_provider(self):
        """
        Getter for __resource_provider

        :return: ResourceProvider
        """
        return self._resources.resource_provider

    @property
    def duts(self):
        """
        Get _duts.

        :return: list
        """
        return self._resources.duts

    @duts.setter
    def duts(self, value):
        """
        set a list as _duts.

        :param value: list
        :return: Nothing
        """
        self._resources.duts = value

    def duts_iterator_all(self):
        """
        Yield indexes and related duts.
        """
        return self._resources.duts_iterator_all()

    def duts_iterator(self):
        """
        Yield indexes and related duts that are for this test case.
        """
        return self._resources.duts_iterator()

    def is_allowed_dut_index(self, dut_index):
        """
        Check if dut_index is one of the duts for this test case.

        :param dut_index: int
        :return: Boolean
        """
        return self._resources.is_allowed_dut_index(dut_index=dut_index)

    @property
    def dut_indexes(self):
        """
        Get a list with dut indexes.

        :return: list
        """
        return self._resources.dut_indexes

    def get_dut(self, k):
        """
        Get dut object.

        :param k: index or nickname of dut.
        :return: Dut
        """
        return self._resources.get_dut(k)

    def get_node_endpoint(self, endpoint_id):
        """
        get NodeEndPoint object for dut endpoint_id.

        :param endpoint_id: nickname of dut
        :return: NodeEndPoint
        """
        return self._resources.get_node_endpoint(endpoint_id, self)

    def is_my_dut_index(self, dut_index):
        """
        :return: Boolean
        """
        return self._resources.is_my_dut_index(dut_index)

    @property
    def dutinformations(self):
        """
        Getter for DutInformation list.

        :return: list
        """
        return self._resources.dutinformations

    @dutinformations.setter
    def dutinformations(self, value):
        """
        Setter for dutinformations

        :param value: DutInformationList
        :return: Nothing
        """
        self._resources.dutinformations = value

    def reset_dut(self, dut_index="*"):
        """
        Reset dut k.

        :param dut_index: index of dut to reset. Default is *, which causes all duts to be reset.
        :return: Nothing
        """
        return self._resources.reset_dut(dut_index)

    def get_dut_nick(self, dut_index):
        """
        Get nick of dut index k.

        :param dut_index: index of dut
        :return: string
        """
        return self._resources.get_dut_nick(dut_index)

    def get_dut_index(self, nick):
        """
        Get index of dut with nickname nick.

        :param nick: string
        :return: integer > 1
        """
        return self._resources.get_dut_index(nick)

    def is_my_dut(self, k):
        """
        :return: Boolean
        """
        return self._resources.is_my_dut(k)

    @staticmethod
    def create_new_result(verdict, retcode, duration, input_data):
        """
        Create a new Result object with data in function arguments.

        :param verdict: Verdict as string
        :param retcode: Return code as int
        :param duration: Duration as time
        :param input_data: Input data as dictionary
        :return: Result
        """
        return Results.create_new_result(verdict, retcode, duration, input_data)

    def add_new_result(self, verdict, retcode, duration, input_data):
        """
        Add a new Result to result object to the internal ResultList.

        :param verdict: Verdict as string
        :param retcode: Return code as int
        :param duration: Duration as time
        :param input_data: Input data as dict
        :return: Result
        """
        return self._resultfunctions.add_new_result(verdict, retcode, duration, input_data)

    @property
    def results(self):
        """
        Getter for internal _results variable.
        """
        return self._resultfunctions.get_results()

    @results.setter
    def results(self, value):
        """
        Call setter for internal ResultList.

        :param value: ResultList
        :return: Nothing
        """
        self._resultfunctions.set_results(value)

    @property
    def retcode(self):
        """
        Getter for return code.

        :return: int
        """
        return self._resultfunctions.retcode

    @retcode.setter
    def retcode(self, value):
        """
        Setter for retcode.

        :param value: int
        :return: Nothing
        """
        self._resultfunctions.retcode = value

    def get_result(self, tc_file=None):
        """
        Generate a Result object from this test case.

        :param tc_file: Location of test case file
        :return: Result
        """
        return self._resultfunctions.get_result(tc_file)

    def append_result(self, tc_file=None):
        """
        Append a new fully constructed Result to the internal ResultList.

        :param tc_file: Test case file path
        :return: Nothing
        """
        return self._resultfunctions.append_result(tc_file)

    def set_failure(self, retcode, reason):
        """
        Set internal state to reflect failure of test.

        :param retcode: return code
        :param reason: failure reason as string
        :return: Nothing
        """
        return self._resultfunctions.set_failure(retcode, reason)

    def input_from_user(self, title=None):
        """
        Input data from user.

        :param title: Title as string
        :return: stripped data from stdin.
        """
        return self._benchfunctions.input_from_user(title)

    def open_node_terminal(self, k="*", wait=True):
        """
        Open Putty (/or kitty if exists)

        :param k: number 1.<max duts> or '*' to open putty to all devices
        :param wait: wait while putty is closed before continue testing
        :return: Nothing
        """
        return self._benchfunctions.open_node_terminal(k, wait)

    def delay(self, seconds):
        """
        Sleep command.

        :param seconds: Amount of seconds to sleep.
        :return: Nothing
        """
        return self._benchfunctions.delay(seconds)

    def verify_trace_skip_fail(self, k, expected_traces):
        """
        Shortcut to set break_in_fail to False in verify_trace.

        :param k: nick or index of dut.
        :param expected_traces: Expected traces as a list or string
        :return: boolean
        """
        return self._benchfunctions.verify_trace_skip_fail(k, expected_traces)

    def verify_trace(self, k, expected_traces, break_in_fail=True):
        """
        Verify that traces expected_traces are found in dut traces.

        :param k: index or nick of dut whose traces are to be used.
        :param expected_traces: list of expected traces or string
        :param break_in_fail: Boolean, if True raise LookupError if search fails
        :return: boolean.
        :raises: LookupError if search fails.
        """
        return self._benchfunctions.verify_trace(k, expected_traces, break_in_fail)

    def get_time(self):
        """
        Get timestamp using time.time().

        :return: timestamp
        """
        return self._benchfunctions.get_time()

    @property
    def command(self):
        """
        Alias for execute_command.

        :return: execute_command attribute reference.
        """
        return self._commands.command

    def wait_for_async_response(self, cmd, async_resp):
        """
        Wait for the given asynchronous response to be ready and then parse it.

        :param cmd: The asynchronous command that was sent to DUT.
        :param async_resp: The asynchronous response returned by the preceding command call.
        :return: CliResponse object
        """
        return self._commands.wait_for_async_response(cmd, async_resp)

    def execute_command(self, k, cmd,  # pylint: disable=invalid-name
                        wait=True,
                        timeout=50,
                        expected_retcode=0,
                        asynchronous=False,
                        report_cmd_fail=True):
        """
        Do Command request for DUT.
        If this fails, testcase will be marked as failed in internal mechanisms.
        This will happen even if you later catch the exception in your testcase.
        To get around this (allow command failing without throwing exceptions),
        use the reportCmdFail parameter which disables the raise.

        :param k: Index where command is sent, '*' -send command for all duts.
        :param cmd: Command to be sent to DUT.
        :param wait: For special cases when retcode is not wanted to wait.
        :param timeout: Command timeout in seconds.
        :param expected_retcode: Expecting this retcode, default: 0, can be None when it is ignored.
        :param asynchronous: Send command, but wait for response in parallel.
        When sending next command previous response will be wait. When using async mode,
        response is dummy.
        :param report_cmd_fail: If True (default), exception is thrown on command execution error.
        :return: CliResponse object
        """
        return self._commands.execute_command(k, cmd,  # pylint: disable=invalid-name
                                              wait,
                                              timeout,
                                              expected_retcode,
                                              asynchronous,
                                              report_cmd_fail)

    def send_post_commands(self, cmds=""):
        """
        Send post commands to duts.

        :param cmds: Commands to send as string.
        :return:
        """
        return self._commands.send_post_commands(cmds)

    def send_pre_commands(self, cmds=""):
        """
        Send pre-commands to duts.

        :param cmds: Commands to send as string
        :return: Nothing
        """
        return self._commands.send_pre_commands(cmds)

    def init_sniffer(self):
        """
        Initialize and start sniffer if it is required.

        :return: Nothing
        """
        return self._nwsniffer.init_sniffer()

    def get_start_time(self):
        """
        Get test start timestamp.
        :return: None or timestamp.
        """
        return self._resources.get_start_time()

    @property
    def wshark(self):
        """
        Return wireshark object.

        :return: Wireshark
        """
        return self._nwsniffer.wshark

    @property
    def tshark_arguments(self):
        """
        Get tshark arguments.

        :return: dict
        """
        return self._nwsniffer.tshark_arguments

    @property
    def sniffer_required(self):
        """
        Check if sniffer was requested for this run.

        :return: Boolean
        """
        return self._nwsniffer.sniffer_required

    def clear_sniffer(self):
        """
        Clear sniffer

        :return: Nothing
        """
        return self._nwsniffer.clear_sniffer()

    @property
    def capture_file(self):
        """
        Return capture file path.

        :return: file path of capture file.
        """
        return self._nwsniffer.capture_file

    def get_nw_log_filename(self):
        """
        Get nw data log file name.

        :return: string
        """
        return self._nwsniffer.get_nw_log_filename()

    @property
    def pluginmanager(self):
        """
        Getter for PluginManager.

        :return: PluginManager
        """
        return self._plugins.pluginmanager

    @pluginmanager.setter
    def pluginmanager(self, value):
        """
        Setter for PluginManager.
        """
        self._plugins.pluginmanager = value

    def start_external_services(self):
        """
        Start ExtApps required by test case.

        :return: Nothing
        """
        return self._plugins.start_external_services()

    def stop_external_services(self):
        """
        Stop external services started via PluginManager
        """
        return self._plugins.stop_external_services()

    def parse_response(self, cmd, response):
        """
        Parse a response for command cmd.

        :param cmd: Command
        :param response: Response
        :return: Parsed response (usually dict)
        """
        return self._plugins.parse_response(cmd, response)

    def _validate_dut_configs(self, dut_configuration_list, logger):
        """
        Validate dut configurations.

        :param dut_configuration_list: dictionary with dut configurations
        :param logger: logger to be used
        :raises EnvironmentError if something is wrong
        """
        return self._resources.validate_dut_configs(dut_configuration_list, logger)

    # Backwards compatibility functions here

    def __wrap_obsoleted_functions(self):
        """
        Replaces obsoleted setup and teardown step names with old ones to provide backwards
        compatibility.

        :return: Nothing
        """
        wrappers = {
            'rampUp': 'setup',
            'setUp': 'setup',
            'rampDown': 'teardown',
            'tearDown': 'teardown'
        }
        for key, value in iteritems(wrappers):
            if hasattr(self, key) and not hasattr(self, value):
                self.logger.warning("%s has been deprecated, please rename it to %s", key, value)
                setattr(self, value, getattr(self, key))

    def get_dut_range(self, i=0):
        """
        get range of length dut_count with offset i.
        :param i: Offset
        :return: range
        """
        return self.resource_configuration.get_dut_range(i)

    @deprecated("Please use test_name property instead.")
    def get_test_name(self):
        """
        Get test case name.

        :return: str
        """
        return self.test_name

    @deprecated("_check_skip has been deprecated. Use check_skip instead.")
    def _check_skip(self):
        """
        Backwards compatibility.
        """
        return self._configurations.check_skip()

    @property
    @deprecated("_dut_count has been deprecated. Use dut_count() instead.")
    def _dut_count(self):
        """
        Backwards compatibility.
        """
        return self.dut_count()

    @property
    @deprecated("_platforms has been deprecated. Please use get_platforms() instead.")
    def _platforms(self):
        """
        Deprecated getter property for platforms.

        :return: list of strings
        """
        return self.get_platforms()

    @property
    @deprecated("_serialnumbers has been deprecated. Please use get_serialnumbers() instead.")
    def _serialnumbers(self):
        """
        Deprecated property for serial numbers.

        :return: list of strings
        """
        return self.get_serialnumbers()

    @property
    @deprecated("Use test_name instead.")
    def name(self):
        """
        Returns name from Configurations.
        """
        return self._configurations.name

    @deprecated("Please don't use this function.")
    def get_dut_versions(self):
        """
        Get nname results and set them to duts.

        :return: Nothing
        """
        return self._resources.get_dut_versions(self._commands)

    @property
    @deprecated("_starttime has been deprecated. Please use get_start_time instead.")
    def _starttime(self):
        """
        Deprecated getter for test start time.

        :return: None or timestamp.
        """
        return self.get_start_time()

    @property
    @deprecated("_results has been deprecated. Please use results instead.")
    def _results(self):
        """
        Deprecated getter for results.

        :return: ResultList
        """
        return self.results

    @_results.setter
    @deprecated("_results has been deprecated. Please use results instead.")
    def _results(self, value):
        """
        Deprecated setter for results.

        :param value: ResultList
        :return: Nothing
        """
        self.results = value

    @property
    @deprecated("_dutinformation has been deprecated. Please use dutinformations instead.")
    def _dutinformations(self):
        """
        Deprecated getter for dut information list.

        :return: DutInformationList
        """
        return self.dutinformations

    @_dutinformations.setter
    @deprecated("_dutinformation has been deprecated. Please use dutinformations instead.")
    def _dutinformations(self, value):
        """
        Deprecated setter for dut information list.

        :param value: DutInformationList
        :return: Nothing
        """
        self.dutinformations = value

    @deprecated("set_args has been deprecated, please use the args property instead.")
    def set_args(self, args):
        """
        Deprecated setter for args.

        :param args:
        :return:
        """
        self.args = args

    @deprecated("get_metadata has been deprecated, please use get_config() instead")
    def get_metadata(self):
        """
        Deprecated getter for test configuration and metadata.

        :return: dict
        """
        return self.get_config()

    @deprecated("get_resource_configuration has been deprecated, please use the "
                "resource_configuration property instead.")
    def get_resource_configuration(self):
        """
        Deprecated getter for resource configuration.
        :return: ResourceConfig
        """
        return self.resource_configuration



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# pylint: disable=missing-docstring,old-style-class,protected-access,no-self-use,invalid-name
# pylint: disable=attribute-defined-outside-init,too-few-public-methods,unused-argument
# pylint: disable=too-many-public-methods

import unittest
import logging
import mock

from icetea_lib.AllocationContext import AllocationContextList
from icetea_lib.ResourceProvider.Allocators.exceptions import AllocationError
from icetea_lib.ResourceProvider.ResourceRequirements import ResourceRequirements
from icetea_lib.ResourceProvider.exceptions import ResourceInitError

from icetea_lib.Plugin.plugins.LocalAllocator.LocalAllocator import LocalAllocator, init_process_dut
from icetea_lib.Plugin.plugins.LocalAllocator.LocalAllocator import init_mbed_dut


@mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.LocalAllocator.DutDetection", create=False)
@mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.LocalAllocator.get_resourceprovider_logger",
            create=True)
class TestVerify(unittest.TestCase):

    def setUp(self):
        self.nulllogger = logging.getLogger("test")
        self.nulllogger.addHandler(logging.NullHandler())

    def test_init_with_no_logger(self, mock_logging, mock_dutdetection):
        dutdetect = mock.Mock()
        mock_dutdetection.return_value = dutdetect
        dutdetect.get_available_devices = mock.MagicMock(return_value=None)
        mock_logger = mock.Mock()
        mock_logging.get_logger = mock.MagicMock(return_value=mock_logger)
        mock_logging.NullHandler = mock.MagicMock()

        LocalAllocator()

        mock_dutdetection.assert_not_called()
        dutdetect.get_available_devices.assert_not_called()
        mock_logging.assert_called_once_with("LocalAllocator", "LAL")

    def test_can_allocate_success(self, mock_logging, mock_dutdetection):
        alloc = LocalAllocator()
        data1 = {"type" : "hardware"}
        self.assertTrue(alloc.can_allocate(data1))

    def test_can_allocate_unknown_type(self, mock_logging, mock_dutdetection):
        dutdetect = mock.Mock()
        mock_dutdetection.return_value = dutdetect
        dutdetect.get_available_devices = mock.MagicMock(return_value=None)

        alloc = LocalAllocator()
        data_unknown_type = {"type" : "unknown"}
        self.assertFalse(alloc.can_allocate(data_unknown_type))

        mock_dutdetection.assert_not_called()
        dutdetect.get_available_devices.assert_not_called()

        data_unknown_type = {"type" : None}
        self.assertFalse(alloc.can_allocate(data_unknown_type))

    def test_can_allocate_missing_type(self, mock_logging, mock_dutdetection):
        alloc = LocalAllocator()
        data_no_type = {"notype": None}
        self.assertFalse(alloc.can_allocate(data_no_type))

    def test_allocate_raises_on_invalid_dut_format(self, mock_logging, mock_dutdetection):
        alloc = LocalAllocator()
        mfunc = mock.MagicMock()
        mfunc.get_dut_configuration = mock.MagicMock()
        mfunc.get_dut_configuration.return_value = "Not a list"
        self.assertRaises(AllocationError, alloc.allocate, mfunc)

    def test_allocate_raises_devices_not_available(self, mock_logging, mock_dutdetection):
        detected = [1]
        mock_dutdetection.get_available_devices = mock.MagicMock(return_value=detected)
        alloc = LocalAllocator()
        mfunc = mock.MagicMock()
        mfunc.get_dut_configuration = mock.MagicMock()
        mfunc.get_dut_configuration.return_value = [{"type": "hardware"}, {"type": "hardware"}]
        with self.assertRaises(AllocationError):
            alloc.allocate(mfunc)

    def test_internal_allocate_no_type_raises(self, mock_logging, mock_dutdetection):
        alloc = LocalAllocator()
        data = {"notype": None}
        self.assertRaises(KeyError, alloc._allocate, data)

    def test_internal_allocate_non_hardware_types_success(self, mock_logging, mock_dutdetection):
        dutdetect = mock.Mock()
        mock_dutdetection.return_value = dutdetect
        dutdetect.get_available_devices = mock.MagicMock(return_value=None)

        alloc = LocalAllocator()
        dut = ResourceRequirements({"type": "process"})
        mfunc = mock.MagicMock()
        mfunc.get_dut_configuration = mock.MagicMock()
        mfunc.get_dut_configuration.return_value = [dut]
        self.assertTrue(alloc.allocate(mfunc))

        mock_dutdetection.assert_not_called()
        dutdetect.get_available_devices.assert_not_called()

    def test_internal_allocate_non_hardware_types_success_without_dutfactory(self, mock_logging,
                                                                             mock_dutdetection):
        alloc = LocalAllocator()
        dut = ResourceRequirements({"type": "process"})
        mfunc = mock.MagicMock()
        mfunc.get_dut_configuration = mock.MagicMock()
        mfunc.get_dut_configuration.return_value = [dut]
        self.assertTrue(alloc.allocate(mfunc))

    def test_internal_allocate_hardware_platform_no_devices_raises_error(self, mock_logging,
                                                                         mock_dutdetection):
        dutdetect = mock.Mock()
        mock_dutdetection.return_value = dutdetect
        dutdetect.get_available_devices = mock.MagicMock(return_value=None)

        alloc = LocalAllocator()
        self.assertRaises(AllocationError, alloc._allocate,
                          ResourceRequirements({"type": "hardware"}))

    def test_inter_alloc_suc_one_hardware_device_with_undef_allowed_platf(self, mock_logging,
                                                                          mock_dutdetection):
        # Allocation should succeed if no allowed_platform defined in dut configuration,
        # and devices are available
        dutdetect = mock.Mock()  # DutDetection instance mock
        mock_dutdetection.return_value = dutdetect
        mock_dutdetection.is_port_usable = mock.MagicMock(return_value=True)

        device = {"state": "unknown", "platform_name": "K64F",
                  "target_id": "ABCDEFG12345", "serial_port": "/dev/serial"}
        dutdetect.get_available_devices = mock.MagicMock(return_value=[device])

        alloc = LocalAllocator()
        dut = ResourceRequirements({"type": "hardware"})
        mfunc = mock.MagicMock()
        mfunc.get_dut_configuration = mock.MagicMock()
        mfunc.get_dut_configuration.return_value = [dut]
        self.assertEqual(len(alloc.allocate(mfunc)), 1)

        dutdetect.get_available_devices.assert_called_once_with()

        # Test correct format of resulting dut configuration
        mock_dutdetection.is_port_usable.assert_called_once_with(device["serial_port"])

    def test_inter_alloc_suc_w_two_hw_allocatable_dev_one_has_unusable_serial(self, mock_logging,
                                                                              mock_dutdetection):
        # Test with two devices, both are allocatable, but the serial port for first is unusable
        dutdetect = mock.Mock()  # DutDetection instance mock
        mock_dutdetection.return_value = dutdetect
        mock_dutdetection.is_port_usable = mock.MagicMock(side_effect=iter([False, True]))

        devices = [{"state": "unknown", "platform_name": "K64F",
                    "target_id": "ABCDEFG12345", "serial_port": "/dev/serial"},
                   {"state": "unknown", "platform_name": "K64F",
                    "target_id": "ABCDEFG12345", "serial_port": "/dev/serial"}]
        dutdetect.get_available_devices = mock.MagicMock(return_value=devices)

        alloc = LocalAllocator()
        dut = ResourceRequirements({"type": "hardware"})
        mfunc = mock.MagicMock()
        mfunc.get_dut_configuration = mock.MagicMock()
        mfunc.get_dut_configuration.return_value = [dut]
        self.assertEqual(len(alloc.allocate(mfunc)), 1)
        self.assertEqual(mock_dutdetection.is_port_usable.call_count, 2)

    def test_inter_alloc_suc_w_two_hw_allocatable_dev_one_has_nonmatch_platf(self, mock_logging,
                                                                             mock_dutdetection):
        # Test with two devices, both are allocatable, but the serial port for first is unusable
        dutdetect = mock.Mock()  # DutDetection instance mock
        mock_dutdetection.return_value = dutdetect
        mock_dutdetection.is_port_usable = mock.MagicMock(return_value=True)

        dut = ResourceRequirements({"type": "hardware", "allowed_platforms": ["K64F"]})
        devices = [{"state": "unknown", "platform_name": "unknown",
                    "target_id": "ABCDEFG12345", "serial_port": "/dev/serial1"},
                   {"state": "unknown", "platform_name": "K64F",
                    "target_id": "ABCDEFG12345", "serial_port": "/dev/serial2"}]
        dutdetect.get_available_devices = mock.MagicMock(return_value=devices)

        alloc = LocalAllocator()
        # Test correct format of resulting dut configuration
        mfunc = mock.MagicMock()
        mfunc.get_dut_configuration = mock.MagicMock()
        mfunc.get_dut_configuration.return_value = [dut]
        self.assertEqual(len(alloc.allocate(mfunc)), 1)

        resultingdut = dut
        resultingdevice = devices[1]
        resultingdevice["state"] = "allocated"
        resultingdut.set("allocated", resultingdevice)

        mock_dutdetection.is_port_usable.assert_called_once_with(resultingdevice["serial_port"])

    def test_inter_alloc_suc_w_two_hw_allocatabl_dev_w_match_platf_one_alloc(self, mock_logging,
                                                                             mock_dutdetection):
        # Test with two devices, both are allocatable, but the serial port for first is unusable
        dutdetect = mock.Mock()  # DutDetection instance mock
        mock_dutdetection.return_value = dutdetect
        mock_dutdetection.is_port_usable = mock.MagicMock(return_value=True)

        dut = ResourceRequirements({"type": "hardware", "allowed_platforms": ["K64F"]})
        devices = [{"state": "allocated", "platform_name": "K64F",
                    "target_id": "ABCDEFG12345", "serial_port": "/dev/serial1"},
                   {"state": "unknown", "platform_name": "K64F",
                    "target_id": "ABCDEFG12345", "serial_port": "/dev/serial2"}]
        dutdetect.get_available_devices = mock.MagicMock(return_value=devices)

        alloc = LocalAllocator()
        # Test correct format of resulting dut configuration
        mfunc = mock.MagicMock()
        mfunc.get_dut_configuration = mock.MagicMock()
        mfunc.get_dut_configuration.return_value = [dut]
        self.assertEqual(len(alloc.allocate(mfunc)), 1)

        resultingdut = dut
        resultingdevice = devices[1]
        resultingdevice["state"] = "allocated"
        resultingdut.set("allocated", resultingdevice)

        mock_dutdetection.is_port_usable.assert_called_once_with(resultingdevice["serial_port"])

    def test_alloc_twice_suc_when_two_dev_available(self, mock_logging, mock_dutdetection):
        # Test with two devices, both are allocatable, but the serial port for first is unusable
        dutdetect = mock.Mock()  # DutDetection instance mock
        mock_dutdetection.return_value = dutdetect
        mock_dutdetection.is_port_usable = mock.MagicMock(return_value=True)

        devices = [{"state": "unknown",
                    "platform_name": "K64F", "target_id": "1234", "serial_port": "/dev/serial1"},
                   {"state": "unknown",
                    "platform_name": "K64F", "target_id": "5678", "serial_port": "/dev/serial2"}]
        dutdetect.get_available_devices = mock.MagicMock(return_value=devices)

        alloc = LocalAllocator()
        duts = [ResourceRequirements({"type": "hardware", "allowed_platforms": ["K64F"]}),
                ResourceRequirements({"type": "hardware", "allowed_platforms": ["K64F"]})]
        mfunc = mock.MagicMock()
        mfunc.get_dut_configuration = mock.MagicMock()
        mfunc.get_dut_configuration.return_value = duts
        self.assertEqual(len(alloc.allocate(mfunc)), 2)

        # Result of first dut allocation
        resultingdut1 = duts[0]
        resultingdevice1 = devices[0]
        resultingdevice1["state"] = "allocated"
        resultingdut1.set("allocated", resultingdevice1)
        # Result of second dut allocation
        resultingdut2 = duts[1]
        resultingdevice2 = devices[1]
        resultingdevice2["state"] = "allocated"
        resultingdut2.set("allocated", resultingdevice2)
        mock_dutdetection.is_port_usable.assert_has_calls(
            [mock.call(resultingdevice1["serial_port"]),
             mock.call(resultingdevice2["serial_port"])])

    def test_alloc_fail_w_two_hw_allocatable_dev_both_already_allocated(self, mock_logging,
                                                                        mock_dutdetection):
        # Allocation should raise AllocationError if no unallocated devices
        dutdetect = mock.Mock()  # DutDetection instance mock
        mock_dutdetection.return_value = dutdetect
        mfunc = mock.MagicMock()
        mfunc.get_dut_configuration = mock.MagicMock()
        mfunc.get_dut_configuration.return_value = [ResourceRequirements({"type": "hardware"})]
        devices = [{"state": "allocated", "platform_name": "K64F",
                    "target_id": "ABCDEFG12345", "serial_port": "/dev/serial"},
                   {"state": "allocated", "platform_name": "K64F",
                    "target_id": "ABCDEFG12345", "serial_port": "/dev/serial"}]
        dutdetect.get_available_devices = mock.MagicMock(return_value=devices)

        alloc = LocalAllocator()
        self.assertRaises(AllocationError, alloc.allocate, mfunc)

    def test_local_allocator_release(self, mock_logging, mock_dutdetection):
        alloc = LocalAllocator()
        alloc.release()

    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.LocalAllocator.DutConsole")
    def test_init_console_dut(self, mock_dc, mock_logging, mock_dutdetection):
        conf = {}
        conf["subtype"] = "console"
        conf["application"] = "stuff"
        con_list = AllocationContextList(self.nulllogger)
        init_process_dut(con_list, conf, 1, mock.MagicMock())

        conf["subtype"] = "other"
        con_list = AllocationContextList(self.nulllogger)
        with self.assertRaises(ResourceInitError):
            self.assertIsNone(init_process_dut(con_list, conf, 1, mock.MagicMock()))

    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.LocalAllocator.DutMbed")
    def test_init_mbed_dut(self, mock_ds, mock_logging, mock_dutdetection):
        conf = {"allocated": {"serial_port": "port", "baud_rate": 115200,
                              "platform_name": "test", "target_id": "id"},
                "application": {"bin": "binary"}}
        args = mock.MagicMock()
        rtscts = mock.PropertyMock(return_value=True)
        s_xonxoff = mock.PropertyMock(return_value=True)
        s_timeout = mock.PropertyMock(return_value=True)
        s_ch_size = mock.PropertyMock(return_value=1)
        s_ch_delay = mock.PropertyMock(return_value=True)
        skip_flash = mock.PropertyMock(return_value=False)
        type(args).skip_flash = skip_flash
        type(args).serial_xonxoff = s_xonxoff
        type(args).serial_rtscts = rtscts
        type(args).serial_timeout = s_timeout
        type(args).serial_ch_size = s_ch_size
        type(args).ch_mode_ch_delay = s_ch_delay

        # Setup mocked dut
        dut1 = mock.MagicMock()
        dut1.close_dut = mock.MagicMock()
        dut1.close_connection = mock.MagicMock()
        dut1.flash = mock.MagicMock()
        dut1.flash.side_effect = [True, False]
        dut1.getInfo = mock.MagicMock(return_value="test")
        type(dut1).index = mock.PropertyMock()

        con_list = AllocationContextList(self.nulllogger)
        with mock.patch.object(con_list, "check_flashing_need") as mock_cfn:
            mock_cfn = mock.MagicMock()
            mock_cfn.return_value = True
            mock_ds.return_value = dut1
            init_mbed_dut(con_list, conf, 1, args)

            with self.assertRaises(ResourceInitError):
                init_mbed_dut(con_list, conf, 1, args)
            dut1.close_dut.assert_called()
            dut1.close_connection.assert_called()

    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.LocalAllocator.DutMbed")
    def test_init_mbed_dut_skip_flash(self, mock_ds, mock_logging, mock_dutdetection):
        conf = {"allocated": {"serial_port": "port", "baud_rate": 115200,
                              "platform_name": "test", "target_id": "id"},
                "application": {"bin": "binary"}}
        args = mock.MagicMock()
        rtscts = mock.PropertyMock(return_value=True)
        s_xonxoff = mock.PropertyMock(return_value=True)
        s_timeout = mock.PropertyMock(return_value=True)
        s_ch_size = mock.PropertyMock(return_value=1)
        s_ch_delay = mock.PropertyMock(return_value=True)
        skip_flash = mock.PropertyMock(return_value=True)
        type(args).skip_flash = skip_flash
        type(args).serial_xonxoff = s_xonxoff
        type(args).serial_rtscts = rtscts
        type(args).serial_timeout = s_timeout
        type(args).serial_ch_size = s_ch_size
        type(args).ch_mode_ch_delay = s_ch_delay

        # Setup mocked dut
        dut1 = mock.MagicMock()
        dut1.close_dut = mock.MagicMock()
        dut1.close_connection = mock.MagicMock()
        dut1.flash = mock.MagicMock()
        dut1.flash.side_effect = [True, False]
        dut1.getInfo = mock.MagicMock(return_value="test")
        type(dut1).index = mock.PropertyMock()

        con_list = AllocationContextList(self.nulllogger)
        with mock.patch.object(con_list, "check_flashing_need") as mock_cfn:
            mock_cfn = mock.MagicMock()
            mock_cfn.return_value = True
            mock_ds.return_value = dut1
            init_mbed_dut(con_list, conf, 1, args)
            dut1.flash.assert_not_called()

    @mock.patch("icetea_lib.Plugin.plugins.LocalAllocator.LocalAllocator.DutMbed")
    def test_init_mbed_dut_nondefault_baud_rate(self, mock_ds, mock_logging, mock_dutdetection):
        conf = {"allocated": {"serial_port": "port", "baud_rate": 115200,
                              "platform_name": "test", "target_id": "id"},
                "application": {"bin": "binary", "baudrate": 9600}}
        args = mock.MagicMock()

        rtscts = mock.PropertyMock(return_value=True)
        s_xonxoff = mock.PropertyMock(return_value=True)
        s_timeout = mock.PropertyMock(return_value=True)
        s_ch_size = mock.PropertyMock(return_value=1)
        s_ch_delay = mock.PropertyMock(return_value=True)
        skip_flash = mock.PropertyMock(return_value=False)
        type(args).skip_flash = skip_flash
        type(args).baudrate = mock.PropertyMock(return_value=False)
        type(args).serial_xonxoff = s_xonxoff
        type(args).serial_rtscts = rtscts
        type(args).serial_timeout = s_timeout
        type(args).serial_ch_size = s_ch_size
        type(args).ch_mode_ch_delay = s_ch_delay

        # Setup mocked dut
        dut1 = mock.MagicMock()
        dut1.close_dut = mock.MagicMock()
        dut1.closeConnection = mock.MagicMock()
        dut1.flash = mock.MagicMock()
        dut1.flash.side_effect = [True, False]
        dut1.getInfo = mock.MagicMock(return_value="test")
        type(dut1).index = mock.PropertyMock()

        con_list = AllocationContextList(self.nulllogger)
        with mock.patch.object(con_list, "check_flashing_need") as mock_cfn:
            mock_cfn = mock.MagicMock()
            mock_cfn.return_value = True
            mock_ds.return_value = dut1
            init_mbed_dut(con_list, conf, 1, args)
            mock_ds.assert_called_once_with(baudrate=9600,
                                            ch_mode_config={'ch_mode_ch_delay': True,
                                                            'ch_mode': True,
                                                            'ch_mode_chunk_size': 1},
                                            config={'application':
                                                        {'bin': 'binary', 'baudrate': 9600},
                                                    'allocated': {'baud_rate': 115200,
                                                                  'platform_name': 'test',
                                                                  'target_id': 'id',
                                                                  'serial_port': 'port'}},
                                            name='D1',
                                            port='port',
                                            serial_config={'serial_timeout': True,
                                                           'serial_rtscts': True},
                                            params=args)


if __name__ == '__main__':
    unittest.main()



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Argument handling mixer for Bench.
"""

from icetea_lib.arguments import get_base_arguments
from icetea_lib.arguments import get_parser
from icetea_lib.arguments import get_tc_arguments


class ArgsHandler(object):
    """
    This mixer get and parses cli arguments
    It brings two API's:
    args -getter and setter
    """
    def __init__(self, **kwargs):
        super(ArgsHandler, self).__init__(**kwargs)
        parser = get_tc_arguments(get_base_arguments(get_parser()))
        self.__args, self.__unknown = parser.parse_known_args()

    @property
    def args(self):
        """
        Property for arguments.

        :return: arguments
        """
        return self.__args

    @args.setter
    def args(self, value):
        """
        Setter for arguments.

        :param value: arguments
        :return: Nothing
        """
        self.__args = value

    @property
    def unknown(self):
        """
        Getter for unknown arguments.
        """
        return self.__unknown

    @unknown.setter
    def unknown(self, value):
        """
        Setter for the unknown variable.
        """
        self.__unknown = value



# pylint: disable=missing-docstring,broad-except,too-few-public-methods,too-many-instance-attributes
# pylint: disable=too-many-arguments,protected-access,too-many-branches,unused-argument
# pylint: disable=too-many-statements,no-member,wrong-import-order
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import inspect
import unittest
import threading
import logging

import mock

from icetea_lib.bench import Bench, ReturnCodes
from icetea_lib.LogManager import BenchLoggerAdapter
from icetea_lib.Plugin.PluginManager import PluginManager
from icetea_lib.ResourceProvider.ResourceConfig import ResourceConfig
from icetea_lib.ResourceProvider.ResourceProvider import ResourceProvider
from icetea_lib.TestStepError import TestStepFail, TestStepError, InconclusiveError, TestStepTimeout
from test.tests.test_tcTearDown import Testcase as TearDownTest


class MockDut(object):
    def __init__(self):
        self.traces = ["this is test line 1", "this is test line 2"]


class TestingTestcase(Bench):
    """
    Testcase class for testing all exception cases
    """
    def __init__(self, teststep_fail=False, teststep_error=False,
                 name_error=False, value_error=False, kbinterrupt=False,
                 exception=False, inconclusive_error=False, in_setup=False,
                 in_case=False, in_teardown=False, test_step_timeout=False):
        self.teststep_error = teststep_error
        self.teststep_fail = teststep_fail
        self.name_error = name_error
        self.value_error = value_error
        self.kbinterrupt = kbinterrupt
        self.exception = exception
        self.in_setup = in_setup
        self.in_case = in_case
        self.in_teardown = in_teardown
        self.inconclusive = inconclusive_error
        self.test_step_timeout = test_step_timeout
        Bench.__init__(self,
                       name="ut_exception",
                       title="unittest exception in testcase",
                       status="development",
                       type="acceptance",
                       purpose="dummy",
                       requirements={
                           "duts": {
                               '*': {  # requirements for all nodes
                                   "count": 0,
                               }
                           }}
                      )

    def raise_exc(self):
        if self.teststep_fail:
            raise TestStepFail("This is a TestStepFail")
        if self.teststep_error:
            raise TestStepError("This is a TestStepError")
        elif self.name_error:
            raise NameError("This is a NameError")
        elif self.value_error:
            raise ValueError("This is a ValueError")
        elif self.exception:
            raise Exception("This is a generic exception")
        elif self.kbinterrupt:
            raise KeyboardInterrupt()
        elif self.inconclusive:
            raise InconclusiveError("This will result in an inconclusive retcode.")
        elif self.test_step_timeout:
            raise TestStepTimeout("This is TestStepTimeout")

    def setup(self):
        self.args.silent = True
        if self.in_setup:
            self.raise_exc()

    def case(self):
        if self.in_case:
            self.raise_exc()

    def teardown(self):
        if self.in_teardown:
            self.raise_exc()


class ApiTestcase(Bench):
    """
    Testcase class for testing all exception cases
    """

    def __init__(self):
        Bench.__init__(self,
                       name="ut_apis",
                       title="unittest apis in testcase",
                       status="development",
                       type="acceptance",
                       purpose="dummy",
                       component=["UT"],
                       feature=["public apis"],
                       requirements={
                           "duts": {
                               '*': {
                                   "count": 0,
                                   "allowed_platforms": ["TEST1"]
                               }
                           }
                       }
                      )
        self.expected_config = {'status': 'development',
                                'component': ["UT"],
                                'feature': ["public apis"],
                                'requirements': {
                                    'duts': {
                                        '*': {
                                            'count': 0,
                                            'application': {
                                                'bin': None
                                            },
                                            "allowed_platforms": ["TEST1"]
                                        }
                                    },
                                    'external': {
                                        'apps': []
                                    }
                                },
                                'name': 'ut_apis',
                                'title': 'unittest apis in testcase',
                                'compatible': {
                                    'framework': {
                                        'version': '>=1.0.0',
                                        'name': 'Icetea'
                                    }, 'hw': {
                                        'value': True
                                    }, 'automation': {
                                        'value': True}
                                }, 'purpose': 'dummy',
                                'type': 'acceptance',
                                'sub_type': None
                               }

    def setup(self):
        pass

    def raise_exc(self, step):  # pylint: disable=no-self-use
        print(step)
        raise TestStepFail(step)

    def case(self):
        if self.test_name != "ut_apis":
            self.raise_exc("test_name broken.")

        if self.config != self.expected_config:
            self.raise_exc("config getter broken.")
        self.expected_config["test"] = "test"
        self.config = self.expected_config

        if self.config != self.expected_config:
            self.raise_exc("config setter broken.")

        if self.env != {'sniffer': {'iface': 'Sniffer'}}:
            self.raise_exc("env broken.")

        if self.is_hardware_in_use():
            self.raise_exc("is_hardware_in_use broken.")

        if self.get_platforms() != list():
            self.raise_exc("get_platforms broken.")

        if self.get_serialnumbers():
            self.raise_exc("get_serialnumbers broken")

        if self.get_test_component() != self.expected_config["component"]:
            self.raise_exc("get_test_component broken")
        if len(self.get_allowed_platforms()) != 1 or self.get_allowed_platforms()[0] != "TEST1":
            self.raise_exc("Allowed platforms broken.")
        if self.status() != self.expected_config['status']:
            self.raise_exc("status broken")
        if self.type() != self.expected_config["type"]:
            self.raise_exc("type broken")
        if self.get_features_under_test() != self.expected_config["feature"]:
            self.raise_exc("features broken")
        if self.subtype() != self.expected_config["sub_type"]:
            self.raise_exc("subtype broken")
        if self.config != self.get_config():
            self.raise_exc("config not the same as get_config()")
        if self.skip() is not None:
            self.raise_exc("skip is not None")
        if self.skip_info() is not None:
            self.raise_exc("skip_info is not None")
        if self.skip_reason() != "":
            self.raise_exc("skip_reason is not empty")
        if self.check_skip() is not False:
            self.raise_exc("check_skip was not False")
        if self.get_tc_abspath(__file__) != __file__:
            self.raise_exc("get_tc_abspath file name did not match")
        self.set_config({"test": "test1"})
        if self.config != {"test": "test1"}:
            self.raise_exc("set_config broken.")
        self.set_config(self.expected_config)
        if self.dut_count() != 0:
            self.raise_exc("dut_count broken.")
        if self.get_dut_count() != 0:
            self.raise_exc("get_dut_count broken.")
        if not isinstance(self.resource_provider, ResourceProvider):
            self.raise_exc("resource_provider broken.")
        if not isinstance(self.resource_configuration, ResourceConfig):
            self.raise_exc("resource_configuration broken.")
        if self.duts != list():
            self.raise_exc("duts broken.")
        self.duts = ["D1"]
        if self.duts != ["D1"]:
            self.raise_exc("duts setter broken.")
        self.duts = []
        if self.dut_indexes:
            self.raise_exc("dut_indexes broken.")
        if not inspect.isgenerator(self.duts_iterator_all()):
            self.raise_exc("duts_iterator_all broken.")
        if not inspect.isgenerator(self.duts_iterator()):
            self.raise_exc("duts_iterator is broken.")
        if self.is_allowed_dut_index(1) is not False:
            self.raise_exc("is_allowed_dut_index broken")
        try:
            self.get_dut(1)
        except ValueError:
            pass
        except Exception:
            self.raise_exc("get_dut is broken.")
        try:
            self.get_node_endpoint(1)
        except ValueError:
            pass
        except Exception:
            self.raise_exc("get_node_endpoint is broken.")
        if not self.is_my_dut_index(1):
            self.raise_exc("is_my_dut_index is broken.")
        if self.dutinformations != list():
            self.raise_exc("dutinformations is broken.")
        if self.get_dut_nick(1) != "1":
            self.raise_exc("get_dut_nick is broken.")
        try:
            self.get_dut_nick("does not exists")
        except KeyError:
            pass
        except Exception:
            self.raise_exc("get_dut_nick is broken.")
        try:
            self.get_dut_index("does not exist")
        except ValueError:
            pass
        except Exception:
            self.raise_exc("get_dut_index is broken.")
        if not self.is_my_dut(1):
            self.raise_exc("is_my_dut broken.")
        if self.results is None:
            self.raise_exc("results broken.")
        if self.retcode != ReturnCodes.RETCODE_SUCCESS:
            self.raise_exc("retcode broken.")
        if self.wshark is not None:
            self.raise_exc("wshark default is broken.")
        if self.tshark_arguments != dict():
            self.raise_exc("tshark_arguments default is broken.")
        if self.sniffer_required is not False:
            self.raise_exc("sniffer_required is broken.")
        if not self.get_nw_log_filename().endswith("network.nw.pcap"):
            self.raise_exc("get_nw_log_filename broken.")
        if not isinstance(self.pluginmanager, PluginManager):
            self.raise_exc("pluginmanager broken.")
        logger = self.get_logger()
        if not isinstance(logger, BenchLoggerAdapter):
            self.raise_exc("get_logger broken.")
        if not isinstance(self.unknown, list):
            self.raise_exc("unknown broken.")
        old_list = self.unknown
        new_list = ["val1"]
        self.unknown = new_list
        if self.unknown is not new_list:
            self.raise_exc("unknown setter broken.")
        self.unknown = old_list
        old_res_conf = self.resource_configuration
        new_res_conf = ["val1"]
        self.resource_configuration = new_res_conf
        if self.resource_configuration is not new_res_conf:
            self.raise_exc("resource_configuration setter broken.")
        self.resource_configuration = old_res_conf
        if self.retcode != 0:
            self.raise_exc("retcode broken.")
        self.retcode = 1
        if self.retcode != 1:
            self.raise_exc("retcode setter broken")
        self.retcode = 0
        try:
            self.append_result()
        except Exception:
            self.raise_exc("append_result broken.")
        self.delay(0.1)
        if not isinstance(self.get_time(), float):
            self.raise_exc("get_time broken.")
        try:
            self.verify_trace_skip_fail(1, "a")
        except IndexError:
            pass
        except Exception:
            self.raise_exc("verify_trace_skip_fail broken.")
        retval = None
        try:
            retval = self.wait_for_async_response("a", "b")
        except AttributeError:
            pass
        except Exception:
            self.raise_exc("wait_for_async_response broken.")
        if retval is not None:
            self.raise_exc("wait_for_async_response broken.")
        try:
            self.execute_command(1, "a")
        except ValueError:
            pass
        except Exception:
            self.raise_exc("execute_command broken.")
        old_pm = self.pluginmanager
        new_pm = ["val1"]
        self.pluginmanager = new_pm
        if self.pluginmanager is not new_pm:
            self.raise_exc("pluginmanager setter broken")
        self.pluginmanager = old_pm
        if self.parse_response("a", "b"):
            self.raise_exc("parse_response broken.")
        if self.get_test_name() != "ut_apis":
            self.raise_exc("get_test_name broken.")
        if self.dut_count() != 0:
            self.raise_exc("dut_count broken.")
        try:
            self.init_duts()
        except Exception:
            self.raise_exc("init_duts broken")
        if self.name != "ut_apis":
            self.raise_exc("name broken.")
        mock_dut = mock.MagicMock()
        type(mock_dut).index = mock.PropertyMock(return_value=0)
        mock_dut.close_dut = mock.MagicMock()
        mock_dut.close_connection = mock.MagicMock()
        try:
            self.sync_cli("1", retries=0)
        except ValueError:
            pass
        else:
            self.raise_exc("sync_cli broken")

    def teardown(self):
        pass


class TestVerify(unittest.TestCase):

    def setUp(self):
        logging.disable(logging.FATAL)

    def test_bench_apis(self):
        testcase = ApiTestcase()
        retcode = testcase.run()
        self.assertEqual(retcode, ReturnCodes.RETCODE_SUCCESS)

    def test_exceptions_in_case(self):
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(exception=True, in_case=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(teststep_fail=True, in_case=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(teststep_error=True, in_case=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(name_error=True, in_case=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(value_error=True, in_case=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(test_step_timeout=True, in_case=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        result = TestingTestcase(kbinterrupt=True, in_case=True).run()
        self.assertNotEqual(result, 0, "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        self.assertTrue(result in ReturnCodes.INCONCLUSIVE_RETCODES,
                        "Test return code not in inconclusive!")
        result = TestingTestcase(inconclusive_error=True, in_case=True).run()
        self.assertNotEqual(result, 0, "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        self.assertTrue(result in ReturnCodes.INCONCLUSIVE_RETCODES,
                        "Test return code not in inconclusive!")

    def test_verify_trace(self):
        bench = Bench()
        bench.duts.append(MockDut())
        bench.verify_trace(1, "this is test line 2")
        bench.verify_trace(1, ["this is test line 2"])
        with self.assertRaises(LookupError):
            bench.verify_trace(1, "This is not found in traces")

    def test_exceptions_in_rampup(self):
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(exception=True, in_setup=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(teststep_fail=True, in_setup=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(teststep_error=True, in_setup=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(name_error=True, in_setup=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(value_error=True, in_setup=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(kbinterrupt=True, in_setup=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        result = TestingTestcase(inconclusive_error=True, in_setup=True).run()
        self.assertNotEqual(result, 0, "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        self.assertTrue(result in ReturnCodes.INCONCLUSIVE_RETCODES,
                        "Test return code not in inconclusive!")

    @mock.patch("test.tests.test_tcTearDown.Testcase.teardown")
    def test_rampdown_called(self, mock_teardown):
        retcode = TearDownTest(teststepfail=True).run()
        self.assertEquals(retcode, 1001)
        self.assertTrue(mock_teardown.called)
        mock_teardown.reset_mock()

        retcode = TearDownTest(teststeperror=True).run()
        self.assertEquals(retcode, 1001)
        self.assertFalse(mock_teardown.called)

        retcode = TearDownTest(exception=True).run()
        self.assertEquals(retcode, 1001)
        self.assertFalse(mock_teardown.called)

        retcode = TearDownTest(teststeptimeout=True).run()
        self.assertEquals(retcode, 1001)
        self.assertFalse(mock_teardown.called)

        # Test tearDown is called when TestStepTimeout raised in test case
        retcode = TearDownTest(teststeptimeout_in_case=True).run()
        self.assertEquals(retcode, 1005)
        self.assertTrue(mock_teardown.called)
        mock_teardown.reset_mock()

        retcode = TearDownTest(name_error=True).run()
        self.assertEquals(retcode, 1001)
        self.assertFalse(mock_teardown.called)

        retcode = TearDownTest(value_error=True).run()
        self.assertEquals(retcode, 1001)
        self.assertFalse(mock_teardown.called)

    def test_exceptions_in_rampdown(self):
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(exception=True, in_teardown=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(teststep_fail=True, in_teardown=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(teststep_error=True, in_teardown=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(name_error=True, in_teardown=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(value_error=True, in_teardown=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        thread_count = threading.active_count()
        self.assertNotEqual(TestingTestcase(kbinterrupt=True, in_teardown=True).run(), 0,
                            "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        result = TestingTestcase(inconclusive_error=True, in_teardown=True).run()
        self.assertNotEqual(result, 0, "Test execution returned success retcode")
        self.assertEqual(thread_count, threading.active_count())
        self.assertTrue(result in ReturnCodes.INCONCLUSIVE_RETCODES,
                        "Test return code not in inconclusive!")

    def test_no_hanging_threads(self):
        thread_count = threading.active_count()
        TestingTestcase().run()
        self.assertEqual(thread_count, threading.active_count())

    @mock.patch('icetea_lib.TestBench.Resources.ResourceFunctions.resource_provider', create=True)
    @mock.patch('icetea_lib.TestBench.Commands.Commands.execute_command', create=True)
    def test_precmds_to_two_duts(self, mock_ec, mock_rp):
        # pylint: disable=no-self-use
        bench = Bench()
        bench._resource_provider = mock.Mock()
        bench.args = mock.MagicMock()
        bench.args.my_duts = mock.MagicMock(return_value=False)
        bench.execute_command = mock.MagicMock()
        mock_resconf = mock.Mock()
        mock_resconf.get_dut_configuration = mock.MagicMock(return_value=[{
            "pre_cmds": ["first", "second"]}, {"pre_cmds": ["first2", "second2"]}])
        bench.resource_configuration = mock_resconf
        mock_resconf.count_duts = mock.MagicMock(return_value=2)
        # Call using mangled name of __send_pre_commands method
        bench.send_pre_commands()
        mock_ec.assert_has_calls([mock.call(1, "first"),
                                  mock.call(1, "second"),
                                  mock.call(2, "first2"),
                                  mock.call(2, "second2")])

        bench.execute_command.reset_mock()
        # Test again with argument cmds
        bench.send_pre_commands("somecommand")
        mock_ec.assert_has_calls([mock.call(1, "first"), mock.call(1, "second"),
                                  mock.call(2, "first2"), mock.call(2, "second2"),
                                  mock.call("*", "somecommand")])

    @mock.patch('icetea_lib.TestBench.Resources.ResourceFunctions.resource_provider', create=True)
    @mock.patch('icetea_lib.TestBench.Commands.Commands.execute_command', create=True)
    def test_postcmds_to_two_duts(self, mock_ec, mock_rp):
        # pylint: disable=no-self-use
        bench = Bench()
        bench._resource_provider = mock.MagicMock()
        bench.execute_command = mock.MagicMock()
        bench.args = mock.MagicMock()
        bench.logger = mock.MagicMock()
        type(bench.args).my_duts = mock.PropertyMock(return_value=False)
        type(bench.args).pause_when_external_dut = mock.MagicMock(return_value=False)
        bench._resources.init(mock.MagicMock())
        bench._commands.init()
        mock_dut1 = mock.MagicMock()
        type(mock_dut1).index = mock.PropertyMock(return_value=1)
        mock_dut2 = mock.MagicMock()
        type(mock_dut2).index = mock.PropertyMock(return_value=2)
        bench._resources.duts = [mock_dut1, mock_dut2]
        mock_resconf = mock.Mock()
        mock_resconf.get_dut_configuration = mock.MagicMock(
            return_value=[{"post_cmds": ["first", "second"]}, {"post_cmds": ["first2", "second2"]}])
        bench.resource_configuration = mock_resconf
        mock_resconf.count_duts = mock.MagicMock(return_value=2)
        bench.send_post_commands()
        mock_ec.assert_has_calls([mock.call(1, "first"), mock.call(1, "second"),
                                  mock.call(2, "first2"), mock.call(2, "second2")])

        mock_ec.reset_mock()
        # Test again with argument cmds
        bench.send_post_commands("somecommand")
        mock_ec.assert_has_calls([mock.call(1, "first"), mock.call(1, "second"),
                                  mock.call(2, "first2"), mock.call(2, "second2"),
                                  mock.call("*", "somecommand")])

    def test_reset_duts(self):
        bench = Bench()
        bench.logger = mock.MagicMock()
        bench.args = mock.MagicMock()
        bench._resources._args = bench.args
        type(bench.args).my_duts = mock.PropertyMock(return_value=False)
        type(bench.args).pause_when_external_dut = mock.MagicMock(return_value=False)
        bench._resources.init(mock.MagicMock())
        mock_dut = mock.MagicMock()
        dutconf = {"reset.return_value": True, "initCLI.return_value": True}
        mock_dut.configure_mock(**dutconf)
        mock_dutrange = range(2)
        mock_duts = [mock_dut, mock_dut]
        bench.duts = mock_duts
        # TODO: This mocking does not work somehow
        with mock.patch.object(bench.resource_configuration, "get_dut_range",
                               return_value=mock_dutrange):
            with mock.patch.object(bench, "is_my_dut_index", return_value=True):
                for method in ["hard", "soft", None]:
                    bench.args.reset = method
                    bench.reset_dut()
                    mock_dut.reset.assert_called_with(method)
                    self.assertEqual(mock_dut.reset.call_count, 2)
                    mock_dut.reset.reset_mock()

    def test_check_skip(self):
        testcase = TestingTestcase()
        testcase.config["requirements"]["duts"]["*"]["type"] = "process"
        testcase.config["execution"] = {"skip": {"value": True, "only_type": "process"}}
        self.assertEqual(testcase.run(), -1)
        self.assertTrue(testcase.get_result().skipped())

    def test_check_skip_invalid_platform(self):  # pylint: disable=invalid-name
        testcase = TestingTestcase()
        testcase.config["requirements"]["duts"]["*"]["allowed_platforms"] = ["K64F"]
        testcase.config["execution"] = {"skip": {"value": True, "platforms": ["K64F", "K65F"]}}
        self.assertEqual(testcase.run(), 0)
        self.assertFalse(testcase.get_result().skipped())

    def test_check_skip_valid_platform(self):  # pylint: disable=invalid-name
        testcase = TestingTestcase()
        testcase.config["requirements"]["duts"]["*"]["allowed_platforms"] = ["K64F"]
        testcase.config["requirements"]["duts"]["*"]["platform_name"] = "K64F"
        testcase.config["execution"] = {"skip": {"value": True, "platforms": ["K64F"]}}
        self.assertEqual(testcase.run(), -1)
        self.assertTrue(testcase.get_result().skipped())

    def test_check_no_skip(self):
        testcase = TestingTestcase()
        testcase.config["execution"] = {"skip": {"value": True}}
        self.assertEqual(testcase.run(), 0)
        self.assertFalse(testcase.get_result().skipped())

    def test_create_new_result(self):
        test_data = dict()
        test_data["reason"] = "this is a reason"
        result = Bench.create_new_result("fail", 1, 10, test_data)
        self.assertTrue(result.failure)

    def test_create_and_add_new_result(self):
        test_data = dict()
        test_data["reason"] = "this is a reason"
        bench = Bench()
        result = bench.add_new_result("fail", 1, 10, test_data)
        self.assertEqual(len(bench.results), 1)
        self.assertEqual(result.fail_reason, "this is a reason")


if __name__ == '__main__':
    unittest.main()



# pylint: disable=missing-docstring

import unittest
import mock

from serial import SerialException, SerialTimeoutException

from icetea_lib.enhancedserial import EnhancedSerial


class EnhancedSerialTests(unittest.TestCase):

    def test_peek(self):
        ens = EnhancedSerial()
        self.assertEqual(ens.peek(), "")
        ens.buf = "test"
        self.assertEqual(ens.peek(), "test")

    def test_readline(self):
        with mock.patch.object(EnhancedSerial, "read") as mock_read:
            ens = EnhancedSerial()
            mock_read.side_effect = ["test\n".encode("utf-8"), "test".encode("utf-8"),
                                     SerialTimeoutException, SerialException, ValueError]
            self.assertEqual(ens.readline(), "test\n")
            self.assertIsNone(ens.readline(timeout=0))
            self.assertIsNone(ens.readline(timeout=0))
            self.assertIsNone(ens.readline(timeout=0))
            self.assertIsNone(ens.readline(timeout=0))



# pylint: disable=no-member,too-many-instance-attributes,too-many-public-methods,protected-access
# pylint: disable=too-many-nested-blocks
"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Helpers for handling resources. Handles initialization of Duts,
releasing Duts, handling resource configuration etc.
"""

import time

from six import string_types

from icetea_lib.ResourceProvider.ResourceProvider import ResourceProvider
from icetea_lib.ResourceProvider.ResourceConfig import ResourceConfig
from icetea_lib.ResourceProvider.exceptions import ResourceInitError
from icetea_lib.DeviceConnectors.DutInformation import DutInformation
from icetea_lib.DeviceConnectors.Dut import DutConnectionError
from icetea_lib.ResourceProvider.Allocators.exceptions import AllocationError
from icetea_lib.TestStepError import TestStepError
from icetea_lib.build import Build
from icetea_lib.tools.NodeEndPoint import NodeEndPoint


class ResourceFunctions(object):
    """
    ResourceFunctions manage test required resources, like DUT's.
    It provide public API's to get individual DUT's object or iterator over all DUT's.
    """
    def __init__(self, args, logger, configuration, **kwargs):
        super(ResourceFunctions, self).__init__(**kwargs)
        self._allocation_context = None
        self._resource_configuration = ResourceConfig()
        self._resource_provider = None
        self._starttime = None
        self._dutinformations = None
        self._commands = None
        self._duts = []
        self._args = args
        self._logger = logger
        self._configuration = configuration

    def init(self, commands, logger=None):
        """
        Initialize ResourceConfig and ResourceProvider

        :return: Nothing
        """
        if logger:
            self._logger = logger
        # Create ResourceProvider object and resolve the resource requirements from configuration
        self._resource_provider = ResourceProvider(self._args)
        # @todo need better way to handle forceflash_once option
        # because provider is singleton instance.
        self._resource_provider.args.forceflash = self._args.forceflash
        self._resource_configuration = ResourceConfig(logger=self._logger)
        self._resource_provider.resolve_configuration(self._configuration.config,
                                                      self._resource_configuration)
        self._commands = commands

    def init_duts(self, benchapi):
        """
        Initialize Duts, and the network sniffer.

        :return: Nothing
        """
        # Validate dut configurations
        self.validate_dut_configs(self.resource_configuration.get_dut_configuration(),
                                  self._logger)
        # Initialize duts
        if self.resource_configuration.count_duts() > 0:
            self._initialize_duts()
            benchapi._nwsniffer.init_sniffer()
        else:
            self._logger.debug("This TC doesn't use DUT's")
        self._starttime = time.time()

    def duts_release(self):
        """
        Release Duts.

        :return: Nothing
        """
        try:
            # try to close node's by nicely by `exit` command
            # if it didn't work, kill it by OS process kill command
            # also close reading threads if any
            self._logger.debug("Close dut connections")
            # pylint: disable=unused-variable
            for i, dut in self.duts_iterator():
                try:
                    dut.close_dut()
                    dut.close_connection()
                except Exception:  # pylint: disable=broad-except
                    # We want to catch all uncaught Exceptions here.
                    self._logger.error("Exception while closing dut %s!",
                                       dut.dut_name,
                                       exc_info=True if not self._args.silent else False)
                finally:
                    if hasattr(self._resource_provider.allocator, "share_allocations"):
                        if getattr(self._resource_provider.allocator, "share_allocations"):
                            pass
                        else:
                            self._logger.debug("Releasing dut {}".format(dut.index))
                            self.resource_provider.allocator.release(dut=dut)
                    else:
                        self._logger.debug("Releasing dut {}".format(dut.index))
                        self.resource_provider.allocator.release(dut=dut)

            self._logger.debug("Close dut threads")

            # finalize dut thread
            for ind, dut in self.duts_iterator():
                while not dut.finished():
                    time.sleep(0.5)
                    self._logger.debug("Dut #%i is not finished yet..", ind)
        except KeyboardInterrupt:
            self._logger.debug("key interrupt")
            for ind, dut in self.duts_iterator():
                dut.kill_received = True
            self._duts_delete()

    def get_start_time(self):
        """
        Get start time.

        :return: None if test has not started, start time stamp fetched with time.time() otherwise.
        """
        return self._starttime

    @property
    def resource_configuration(self):
        """
        Getter for __resource_configuration.

        :return: ResourceConfig
        """
        return self._resource_configuration

    @resource_configuration.setter
    def resource_configuration(self, value):
        """
        Setter for __resource_configuration.

        :param value: ResourceConfig
        :return: Nothing
        """
        self._resource_configuration = value

    def dut_count(self):
        """
        Getter for dut count from resource configuration.

        :return: int
        """
        if self.resource_configuration:
            return self.resource_configuration.count_duts()
        return 0

    def get_dut_count(self):
        """
        Get dut count.

        :return: int
        """
        return self.dut_count()

    @property
    def resource_provider(self):
        """
        Getter for __resource_provider

        :return: ResourceProvider
        """
        return self._resource_provider

    @property
    def duts(self):
        """
        Get _duts.

        :return: list
        """
        return self._duts

    @duts.setter
    def duts(self, value):
        """
        set a list as _duts.

        :param value: list
        :return: Nothing
        """
        self._duts = value

    @property
    def dut_indexes(self):
        """
        Get a list with dut indexes.

        :return: list
        """
        return range(1, self._resource_configuration.count_duts() + 1)

    def _duts_delete(self):
        """
        Reset internal __duts list to empty list.

        :return: Nothing
        """
        self._logger.debug("delete duts")
        self._duts = []

    def duts_iterator_all(self):
        """
        Yield indexes and related duts.
        """
        for ind, dut in enumerate(self.duts):
            yield ind, dut

    def duts_iterator(self):
        """
        Yield indexes and related duts that are for this test case.
        """
        for ind, dut in enumerate(self.duts):
            if self.is_my_dut_index(ind):
                yield ind, dut

    def is_allowed_dut_index(self, dut_index):
        """
        Check if dut_index is one of the duts for this test case.

        :param dut_index: int
        :return: Boolean
        """
        return dut_index in self.dut_indexes

    def get_dut(self, k):
        """
        Get dut object.

        :param k: index or nickname of dut.
        :return: Dut
        """
        dut_index = k
        if isinstance(k, str):
            dut_index = self.get_dut_index(k)

        if dut_index > len(self.duts) or dut_index < 1:
            self._logger.error("Invalid DUT number")
            raise ValueError("Invalid DUT number when calling get_dut(%i)" % dut_index)
        return self.duts[dut_index - 1]

    def get_node_endpoint(self, endpoint_id, bench):
        """
        get NodeEndPoint object for dut endpoint_id.

        :param endpoint_id: nickname of dut
        :return: NodeEndPoint
        """
        if isinstance(endpoint_id, string_types):
            endpoint_id = self.get_dut_index(endpoint_id)
        return NodeEndPoint(bench, endpoint_id)

    def is_my_dut_index(self, dut_index):
        """
        :return: Boolean
        """
        if self._args.my_duts:
            myduts = self._args.my_duts.split(',')
            if str(dut_index) in myduts:
                return True
            return False
        else:
            return True

    @property
    def dutinformations(self):
        """
        Getter for DutInformation list.

        :return: list
        """
        if self._allocation_context:
            return self._allocation_context.get_dutinformations()
        return list()

    @dutinformations.setter
    def dutinformations(self, value):
        if self._allocation_context:
            self._allocation_context.dutinformations = value

    def reset_dut(self, dut_index='*'):
        """
        Reset dut k.

        :param dut_index: index of dut to reset. Default is *, which causes all duts to be reset.
        :return: Nothing
        """
        if dut_index == '*':
            for ind in self.resource_configuration.get_dut_range():
                if self.is_my_dut_index(ind):
                    self.reset_dut(ind)
            return
        method = None
        if self._args.reset == "hard" or self._args.reset == "soft":
            self._logger.debug("Sending reset %s to dut %d", self._args.reset, dut_index - 1)
            method = self._args.reset
        self.duts[dut_index - 1].init_wait_register()
        self.duts[dut_index - 1].reset(method)
        self._logger.debug("Waiting for dut %d to initialize", dut_index)
        result = self.duts[dut_index - 1].wait_init()
        if not result:
            self._logger.warning("Cli initialization trigger not found. Maybe your application"
                                 " started before we started reading? Try adding --reset"
                                 " to your run command.")
            raise DutConnectionError("Dut cli failed to initialize within set timeout!")
        if self._args.sync_start:
            self._commands.sync_cli(dut_index)
        self._logger.debug("CLI initialized")
        self.duts[dut_index - 1].init_cli()

    def _open_dut_connections(self, allocations):
        """
        Internal helper. Registers waiting for cli initialization and handles the wait
        as well as opens connections.
        """
        for dut in self._duts:
            dut.init_wait_register()

        try:
            allocations.open_dut_connections()
        except DutConnectionError:
            self._logger.exception("Error while opening DUT connections!")
            for dut in self._duts:
                dut.close_dut()
                dut.close_connection()
            raise

        for ind, dut in self.duts_iterator():
            self._logger.info("Waiting for dut %d to initialize.", ind + 1)
            res = dut.wait_init()
            if not res:
                self._logger.warning("Cli initialization trigger not found. Maybe your application"
                                     " started before we started reading? Try adding --reset"
                                     " to your run command.")
                raise DutConnectionError("Dut cli failed to initialize within set timeout!")
            if self._args.sync_start:
                self._logger.info("Synchronizing the command line interface.")
                try:
                    self._commands.sync_cli(dut.index)
                except TestStepError:
                    raise DutConnectionError("Synchronized start for dut {} failed!".format(
                        dut.index))

    def _alloc_error_helper(self):
        """
        Helper for exception handling in the __init_duts method.
        """
        d_info_list = []
        for i, resource in enumerate(self.resource_configuration.get_dut_configuration()):
            dutinfo = DutInformation(resource.get("platform_name"), None, i)
            d_info_list.append(dutinfo)
        self._dutinformations = d_info_list

    def get_platforms(self):
        """
        Get list of dut platforms.

        :return: list
        """
        plat_list = []
        for info in self.dutinformations:
            plat_list.append(info.platform)
        return plat_list

    def get_serialnumbers(self):
        """
        Get list of dut serial numbers.

        :return: list
        """
        serial_number_list = []
        for info in self.dutinformations:
            serial_number_list.append(info.resource_id)
        return serial_number_list

    # Internal function to Initialize cli dut's
    def _initialize_duts(self):
        """
        Internal function to initialize duts

        :return: Nothing
        :raises: DutConnectionError if correct amount of duts were not initialized or if reset
        failed or if cli initialization wait loop timed out.
        """

        # Initialize command line interface
        self._logger.info("Initialize DUT's connections")

        try:
            allocations = self.resource_provider.allocate_duts(self.resource_configuration)
        except (AllocationError, ResourceInitError):
            self._alloc_error_helper()
            raise
        self._allocation_context = allocations
        allocations.set_logger(self._logger)
        allocations.set_resconf(self.resource_configuration)

        try:
            self._duts = allocations.init_duts(args=self._args)

            if len(self._duts) != self.resource_configuration.count_duts():
                raise AllocationError("Unable to initialize required amount of duts.")
        except AllocationError:
            self._alloc_error_helper()
            raise

        self._open_dut_connections(allocations)

        for ind, dut in self.duts_iterator():
            dut.Testcase = self._configuration.name
            dut.init_cli()
            self._logger.debug("DUT[%i]: Cli initialized.", ind)

        for ind, dut in self.duts_iterator():
            self._logger.debug("DUT[%i]: %s", ind, dut.comport)

        self._logger.debug("Initialized %d %s "
                           "for this testcase.", len(self._duts),
                           "dut" if len(self._duts) == 1 else "duts")

    def validate_dut_configs(self, dut_configuration_list, logger):
        """
        Validate dut configurations.

        :param dut_configuration_list: dictionary with dut configurations
        :param logger: logger to be used
        :raises EnvironmentError if something is wrong
        """
        # for now we validate only binaries - if it exists or not.
        if not self._args.skip_flash:
            for conf in dut_configuration_list:
                try:
                    binar = conf.get("application").get("bin")
                    if binar:
                        build = Build.init(binar)
                        if not build.is_exists():
                            logger.warning("Binary '{}' not found".format(binar))
                            raise EnvironmentError("Binary not found")
                except(KeyError, AttributeError):
                    pass

        if logger is not None:
            logger.debug("Configurations seems to be ok")

    def get_dut_versions(self, commands):
        """
        Get nname results and set them to duts.

        :return: Nothing
        """
        resps = commands.command('*', "nname")
        for i, resp in enumerate(resps):
            self.duts[i].version = resp.parsed

    def get_dut_nick(self, dut_index):
        """
        Get nick of dut index k.

        :param dut_index: index of dut
        :return: string
        """
        nick = str(dut_index)
        int_index_in_duts = dut_index in self._configuration.config["requirements"]["duts"]
        str_index_in_duts = False
        if not int_index_in_duts:
            str_index_in_duts = nick in self._configuration.config["requirements"]["duts"]
        if str_index_in_duts:
            nick_in_indexed_reqs = "nick" in self._configuration.config[
                "requirements"]["duts"][nick]
        elif int_index_in_duts:
            nick_in_indexed_reqs = "nick" in self._configuration.config[
                "requirements"]["duts"][dut_index]
        else:
            nick_in_indexed_reqs = False
        if int_index_in_duts and nick_in_indexed_reqs:
            return self._configuration.config["requirements"]["duts"][dut_index]['nick']
        elif str_index_in_duts and nick_in_indexed_reqs:
            return self._configuration.config["requirements"]["duts"][nick]['nick']
        return nick

    def get_dut_index(self, nick):
        """
        Get index of dut with nickname nick.

        :param nick: string
        :return: integer > 1
        """
        for dut_index, dut in enumerate(self.resource_configuration.get_dut_configuration()):
            nickname = dut.get("nick")
            if nickname and nickname == nick:
                return dut_index + 1
        raise ValueError("Cannot find DUT by nick '%s'" % nick)

    def is_my_dut(self, k):
        """
        :return: Boolean
        """
        if self._args.my_duts:
            myduts = self._args.my_duts.split(',')
            if str(k) in myduts:
                return True
            return False
        else:
            return True



"""
Copyright 2017 ARM Limited
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import os

from icetea_lib.tools.file import FileUtils
from icetea_lib.tools.tools import initLogger
# pylint: disable=too-many-arguments


class JsonFile(object):
    """
    JsonFile class, for generating and handling json files.
    """
    def __init__(self, logger=None, filepath=None, filename=None):
        self.logger = logger if logger else initLogger("JsonFile")
        self.filepath = filepath if filepath else os.path.sep
        self.filename = filename if filename else "default_file.json"

    def write_file(self, content, filepath=None, filename=None, indent=None, keys_to_write=None):
        '''
        Write a Python dictionary as JSON to a file.

        :param content: Dictionary of key-value pairs to save to a file
        :param filepath: Path where the file is to be created
        :param filename: Name of the file to be created
        :param indent: You can use this to specify indent level for pretty printing the file
        :param keys_to_write: array of keys that are to be picked from data and written to file.
                Default is None, when all data is written to file.
        :return: Path of file used
        :raises OSError, EnvironmentError, ValueError
        '''
        path = filepath if filepath else self.filepath
        name = filename if filename else self.filename
        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except OSError as error:
                self.logger.error("Error while creating directory: {}".format(error))
                raise

        name = self._ends_with(name, ".json")
        path = self._ends_with(path, os.path.sep)

        if keys_to_write:
            data_to_write = {}
            for key in keys_to_write:
                data_to_write[key] = content[key]
        else:
            data_to_write = content

        try:
            indent = indent if indent else 2
            self._write_json(path, name, 'w', data_to_write, indent)
            return os.path.join(path, name)
        except EnvironmentError as error:
            self.logger.error("Error while opening or writing to file: {}".format(error))
            raise
        except ValueError:
            raise

    def read_file(self, filepath=None, filename=None):
        """
        Tries to read JSON content from filename and convert it to a dict.

        :param filepath: Path where the file is
        :param filename: File name
        :return: Dictionary read from the file
        :raises EnvironmentError, ValueError
        """
        name = filename if filename else self.filename
        path = filepath if filepath else self.filepath
        name = self._ends_with(name, ".json")
        path = self._ends_with(path, os.path.sep)

        try:
            return self._read_json(path, name)
        except EnvironmentError as error:
            self.logger.error("Error while opening or reading the file: {}".format(error))
            raise
        except ValueError as error:
            self.logger.error("File contents cannot be decoded to JSON: {}".format(error))
            raise

    def read_value(self, key, filepath=None, filename=None):
        """
        Tries to read the value of given key from JSON file filename.

        :param filepath: Path to file
        :param filename: Name of file
        :param key: Key to search for
        :return: Value corresponding to given key
        :raises OSError, EnvironmentError, KeyError
        """
        path = filepath if filepath else self.filepath
        name = filename if filename else self.filename
        name = self._ends_with(name, ".json")
        path = self._ends_with(path, os.path.sep)

        try:
            output = self._read_json(path, name)
            if key not in output:
                raise KeyError("Key '{}' not found in file {}".format(key, filename))
            else:
                return output[key]
        except EnvironmentError as error:
            self.logger.error("Error while opening or reading the file: {}".format(error))
            raise

    def write_values(self, data, filepath=None, filename=None, indent=None, keys_to_write=None):
        """
        Tries to write extra content to a JSON file.

        Creates filename.temp with updated content, removes the old file and
        finally renames the .temp to match the old file.
        This is in effort to preserve the data in case of some weird errors cause problems.

        :param filepath: Path to file
        :param filename: Name of file
        :param data: Data to write as a dictionary
        :param indent: indent level for pretty printing the resulting file
        :param keys_to_write: array of keys that are to be picked from data and written to file.
        Default is None, when all data is written to file.
        :return: Path to file used
        :raises EnvironmentError ValueError
        """

        name = filename if filename else self.filename
        path = filepath if filepath else self.filepath
        name = self._ends_with(name, ".json")
        path = self._ends_with(path, os.path.sep)

        if not os.path.isfile(path + name):
            try:
                return self.write_file(data, path, name, indent, keys_to_write)
            except EnvironmentError as error:
                self.logger.error("Error while opening or writing to file: {}".format(error))
                raise
            except ValueError:
                raise

        if keys_to_write:
            data_to_write = {}
            for key in keys_to_write:
                data_to_write[key] = data[key]
        else:
            data_to_write = data

        try:
            with open(path + name, 'r') as fil:
                output = json.load(fil)
                self.logger.info("Read contents of {}".format(filename))
                for key in data_to_write:
                    try:
                        output[key] = data_to_write[key]
                    except TypeError as error:
                        self.logger.error(
                            "File contents could not be serialized into a dict. {}".format(error))
                        raise
            self._write_json(path, name + ".temp", "w", output, indent)
            FileUtils.remove_file(name, path)
            FileUtils.rename_file(name + '.temp', name, path)
            return os.path.join(path, name)
        except EnvironmentError as error:
            self.logger.error(
                "Error while writing to, opening or reading the file: {}".format(error))
            raise
        except ValueError as error:
            self.logger.error(
                "File could not be decoded to JSON. It might be empty? {}".format(error))
            try:
                self._write_json(path, name, "w", data_to_write, indent)
                return os.path.join(path, name)
            except EnvironmentError:
                raise

    def _write_json(self, filepath, filename, writemode, content, indent):
        """
        Helper for writing content to a file.

        :param filepath: path to file
        :param filename: name of file
        :param writemode: writemode used
        :param content: content to write
        :param indent: value for dump indent parameter.
        :return: Norhing
        """
        with open(os.path.join(filepath, filename), writemode) as fil:
            json.dump(content, fil, indent=indent)
            self.logger.info("Wrote content to file {}".format(filename))

    def _read_json(self, path, name):
        """
        Load a json into a dictionary from a file.

        :param path: path to file
        :param name: name of file
        :return: dict
        """
        with open(os.path.join(path, name), 'r') as fil:
            output = json.load(fil)
            self.logger.info("Read contents of {}".format(name))
            return output

    def _ends_with(self, string_to_edit, end):  # pylint: disable=no-self-use
        """
        Check if string ends with characters in end, if not merge end to string.

        :param string_to_edit: string to check and edit.
        :param end: str
        :return: string_to_edit or string_to_edit + end
        """
        if not string_to_edit.endswith(end):
            return string_to_edit + end
        return string_to_edit
